package com.ruoyi.web.controller.app;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.PaymentResult;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.SendAliSmsUtil;
import com.ruoyi.common.utils.SendSmsVo;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.map.GpsCoordinateUtils;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.AsDeviceVO;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.mapper.EtOrderMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.refund.model.Refund;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

/**
 * app接口（不需要登录的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/app")
public class AppController extends BaseController
{
    @Autowired
    private IAsArticleService asArticleService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtParkingAreaService etParkingAreaService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IAsArticleClassifyService asArticleClassifyService;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private AsUserMapper asUserMapper;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private SysLoginService loginService;

    @Autowired
    private IEtHardwareVersionService etHardwareVersionService;

    @Autowired
    private IEtCouponService etCouponService;

    @Resource
    private EtOrderMapper etOrderMapper;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode}")
    private String templateCode;

    @Value("${aliyun.templateCode2}")
    private String templateCode2;

    @Value("${aliyun.templateCode3}")
    private String templateCode3;

    @Value("${aliyun.phone}")
    private String phone;



    /**
     * 根据查询文章分类列表
     */
    @GetMapping("/classify/list")
    public AjaxResult classifyList(AsArticleClassify asArticleClassify)
    {
        logger.info("查询分类列表：【asArticleClassify="+asArticleClassify+"】");
        List<AsArticleClassify> asArticleClassifies = asArticleClassifyService.selectClassifyList(asArticleClassify);
        return success(asArticleClassifies);
    }

    /**
     * 查询文章列表
     */
    @GetMapping("/article/list")
    public TableDataInfo articleList(EtArticle etArticle)
    {
        startPage();
        List<EtArticle> list = asArticleService.selectAsArticleListByApp(etArticle);
        return getDataTable(list);
    }

    /**
     * 获取文章详细信息
     */
    @GetMapping(value = "/article/{articleId}")
    public AjaxResult getArticleInfo(@PathVariable("articleId") Long articleId)
    {
        return success(asArticleService.selectAsArticleByArticleId(articleId));
    }


    /**
     * 查询运营区列表
     */
    @GetMapping("/area/list")
    public TableDataInfo list(EtOperatingArea etOperatingArea)
    {
        etOperatingArea.setStatus("0");//运营区状态：0-运营中；1-停运
        startPage();
        List<EtOperatingArea> list = etOperatingAreaService.selectEtOperatingAreaList(etOperatingArea);
        return getDataTable(list);
    }

    /**
     * 获取运营区详细信息
     */
    @GetMapping(value = "/area/{areaId}")
    public AjaxResult getInfo(@PathVariable("areaId") Long areaId)
    {
        if(areaId == null){
            logger.info("没有传入areaId参数：【areaId={}】",areaId);
            return error("请传areaId参数"+"【areaId="+areaId+"】");
        }
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        return success(area);
    }


    /**
     * 根据定位获取运营区详细信息
     */
    @GetMapping(value = "/area/info")
    public AjaxResult areaInfo(String longitude,String latitude, String deptId,String sn)
    {
        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】",longitude,latitude);
            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
        }
        EtOperatingArea area = etOperatingAreaService.getAreaInfoByLocation(longitude,latitude,deptId,sn);
        return success(area);
    }

    /**
     * 查询停车区列表
     */
    @GetMapping("/parking/list")
    public TableDataInfo list(EtParkingArea etParkingArea)
    {
        etParkingArea.setStatus("0");
//        startPage();
        List<EtParkingArea> list = etParkingAreaService.selectEtParkingAreaList(etParkingArea);
        return getDataTable(list);
    }

    /**
     * 获取停车区详细信息
     */
    @GetMapping(value = "/parking/{parkingId}")
    public AjaxResult getParkingInfo(@PathVariable("parkingId") Long parkingId)
    {
        return success(etParkingAreaService.selectEtParkingAreaByParkingId(parkingId));
    }


    /**
     * 根据经纬度查询附近500米的所有车辆
     */
    @GetMapping(value = "/vehicleLocalization")
    public AjaxResult vehicleLocalization(String longitude,String latitude,String deptId)
    {
        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】",longitude,latitude);
            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
        }
        List<AsDevice> asDevices = asDeviceService.vehicleLocalization(longitude,latitude,deptId);
        return success(asDevices);
    }

    /**
     * 根据运营区id获取所有车辆
     */
    @GetMapping(value = "/allVehicleByArea")
    public AjaxResult allVehicleByArea(Long areaId)
    {
        if(areaId == null){
            logger.info("没有传入areaId参数：【areaId={}】",areaId);
            return error("请传areaId参数"+"【areaId="+areaId+"】");
        }
        List<AsDevice> asDevices = asDeviceService.allVehicleByArea(areaId);
        return success(asDevices);
    }


    /**
     * 根据运营区获取收费方式列表
     */
    @GetMapping("/fee/list")
    public AjaxResult feeList(EtFeeRule etFeeRule)
    {
        List<EtFeeRule> list = null;
        Long modelId = etFeeRule.getModelId();
        if(modelId != null){
            list = etFeeRuleService.selectRuleInfoListByModelId(etFeeRule.getModelId());
        }else{
            list = etFeeRuleService.selectEtFeeRuleListByAreaId(etFeeRule.getAreaId());
        }
        return success(list);
    }

    /**
     * 获取计费套餐详细信息
     */
    @GetMapping(value = "/fee/{ruleId}")
    public AjaxResult getFeeInfo(@PathVariable("ruleId") Long ruleId)
    {
        return success(etFeeRuleService.selectEtFeeRuleByRuleId(ruleId));
    }

//    /**
//     * 获取计费规则  多少分钟内  里程数
//     */
//    @GetMapping(value = "/fee/rule")
//    public AjaxResult getRule()
//    {
//        String appointmentServiceFee = sysConfigService.selectConfigByKey("appointment.service.fee");//预约服务费
//        String dispatchFee = sysConfigService.selectConfigByKey("dispatch.fee");//调度费
//        String vehicleManagementFee = sysConfigService.selectConfigByKey("vehicle.management.fee");//车辆停车点外调度费
//        String startingPrice = sysConfigService.selectConfigByKey("starting.price");//起步价
//        String startingHowManyMinutes = sysConfigService.selectConfigByKey("starting.how.many.minutes");//多少分钟内按起步价
//        String timeFee = sysConfigService.selectConfigByKey("time.fee");//时长费
//        String timeMinutes = sysConfigService.selectConfigByKey("time.minutes");//时长分钟
//        String timeoutMinutes = sysConfigService.selectConfigByKey("timeout.minutes");//预约超时保留分钟
//        String deposit = sysConfigService.selectConfigByKey("deposit");//押金
//        String isMeal = sysConfigService.selectConfigByKey("is.meal");//是否有套餐 是否有套餐：0-没有套餐；1-有套餐
//        String cappingFee = sysConfigService.selectConfigByKey("order.capping.fee");//24小时内封顶费用
//        String forceParkingArea = sysConfigService.selectConfigByKey("force.parking.area");//是否强制在停车区内还车
//        FeeRuleVo build = FeeRuleVo.builder()
//                .appointmentServiceFee(appointmentServiceFee)
//                .dispatchFee(dispatchFee)
//                .vehicleManagementFee(vehicleManagementFee)
//                .startingPrice(startingPrice)
//                .startingHowManyMinutes(startingHowManyMinutes)
//                .timeFee(timeFee)
//                .timeMinutes(timeMinutes)
//                .timeoutMinutes(timeoutMinutes)
//                .deposit(deposit)
//                .isMeal(isMeal)
//                .cappingFee(cappingFee)
//                .forceParkingArea(forceParkingArea)
//                .build();
//        return success(build);
//    }


    /**
     * 响铃寻车
     */
    @PostMapping("/device/ring")
    public AjaxResult ring(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
       Boolean i =asDeviceService.ring(sn);
        return success(i);
    }

    /**
     * 响铃寻车用mac
     */
    @PostMapping("/device/ringByMac")
    public AjaxResult ringByMac(String mac)
    {
        if(StrUtil.isBlank(mac)){
            logger.info("没有mac号参数：【mac={}】",mac);
            return error("请传mac号参数"+"【mac="+mac+"】");
        }
        Boolean i =asDeviceService.ringByMac(mac);
        return success(i);
    }


    /**
     * 根据sn号查询车辆实时信息
     */
    @GetMapping("/device/info")
    public AjaxResult deviceInfo(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        AsDevice device = asDeviceService.selectAsDeviceBySn(sn);
        return success(device);
    }

//    /**
//     * 根据mac 查询设备信息
//     */
//    @GetMapping("/getDeviceInfoByMac")
//    public AjaxResult getDeviceInfoByMac(String mac)
//    {
//        if(StrUtil.isBlank(mac)){
//            logger.info("没有mac号参数：【mac={}】",mac);
//            return error("未传mac号参数");
//        }
//        AsDevice device = asDeviceService.selectAsDeviceByMac(mac);
//        return success(device);
//    }

    /**
     * 根据mac数据查询设备信息
     */
    @GetMapping("/getDeviceInfoByMacList")
    public AjaxResult getDeviceInfoByMacList(String[] macList)
    {
        if(ObjectUtil.isNull(macList)){
            logger.info("没有mac号数组参数：【macList={}】",macList);
            return error("未传mac号参数");
        }
        ArrayList<AsDeviceVO> asDevices = new ArrayList<>();
        for (String mac:macList) {
            AsDeviceVO device = asDeviceService.selectAsDeviceByMac(mac);
            if(ObjectUtil.isNotNull(device)){
                asDevices.add(device);
            }
        }
        return success(asDevices);
    }

    /**
     * 查询当前用户是否有正在进行中的订单
     */
    @PostMapping("/user/isInOrder")
    public AjaxResult isInOrder(Long userId)
    {
        if(StrUtil.isBlank(userId.toString())){
            logger.info("没有userId参数：【userId={}】",userId);
            return error("请传userId参数"+"【userId="+userId+"】");
        }
        List<EtOrder> orders = etOrderService.isInOrder(userId,null);
        return success(orders);
    }

    /**
     * 查询当前用户是否有正在审核中的订单
     */
    @PostMapping("/user/isInAuditOrder")
    public AjaxResult isInAuditOrder(Long userId)
    {
        if(StrUtil.isBlank(userId.toString())){
            logger.info("没有userId参数：【userId={}】",userId);
            return error("请传userId参数"+"【userId="+userId+"】");
        }
        List<EtOrder> orders = etOrderService.isInAuditOrder(userId,null);
        return success(orders);
    }


    /**
     * 根据定位获取地址    逆地理编码
     */
    @GetMapping("/getAddress")
    public AjaxResult deviceList(String location)
    {
//        logger.info("根据定位获取地址：【location="+location+"】");
//        String address = CommonUtil.getAddressByGeo(location);
//        logger.info("根据定位获取地址：【address="+address+"】");
        return AjaxResult.success();
    }

    /**
     * 获取省市县
     */
    @GetMapping("/getDistrictList")
    public AjaxResult getDistrictList()
    {
        logger.info("【获取省市县】");
        String address = CommonUtil.getDistrictList();
//        logger.info("【获取省市县】【address="+address+"】");
        return AjaxResult.success("操作成功", address);
    }

    /**
     * 查询退款是否成功
     */
    @GetMapping("/queryByOutRefundNo")
    public AjaxResult queryByOutRefundNo(Long areaId,String outRefundNo)
    {
        logger.info("查询退款是否成功：【outRefundNo="+outRefundNo+"】");
        Refund refund = wxPayService.queryByOutRefundNo(outRefundNo);
        return AjaxResult.success(refund);
    }

    /**
     * 通过商户订单号查询订单信息
     */
    @GetMapping("/queryOrderByOutTradeNo")
    public AjaxResult queryOrderByOutTradeNo(String outTradeNo)
    {
        logger.info("查询支付订单信息：【outTradeNo="+outTradeNo+"】");
        PaymentResult paymentResult = wxPayService.queryOrderByOutTradeNo(outTradeNo);
        return AjaxResult.success(paymentResult);
    }

    /**
     * 删除订单
     */
    @DeleteMapping("/{orderNo}")
    public AjaxResult remove(@PathVariable String orderNo)
    {
        return toAjax(etOrderService.deleteEtOrderByOrderNo(orderNo));
    }

    /**
     * 根据mac号判断是否有绑定过
     */
    @GetMapping("/isBandByMac")
    public AjaxResult isBandByMac(String mac)
    {
        AjaxResult ajax = AjaxResult.success();
        logger.info("根据mac号判断是否有绑定过：【mac="+mac+"】");
        if(StrUtil.isBlank(mac)){
            throw new ServiceException("mac号不能为空");
        }
        ajax.put(AjaxResult.DATA_TAG,asDeviceService.isBandByMac(mac));
        return ajax;
    }

    /**
     * 计算订单距离
     */
    @GetMapping("/calculateDistance")
    public AjaxResult calculateDistance()
    {
        EtOrder order = new EtOrder();
        order.setType("1");
        List<EtOrder> orders = etOrderService.selectEtOrderList(order);
        for(EtOrder etOrder:orders){
            String tripRouteStr = etOrder.getTripRouteStr();
            if(StrUtil.isNotBlank(tripRouteStr)){
                double[][] doubles = GeoUtils.parseJsonTrack(tripRouteStr);
                double v = GeoUtils.calculateTotalDistance(doubles);
                etOrder.setDistance((int)Math.round(v));
                logger.info("【计算订单距离】,更新订单信息，{}",JSON.toJSON(etOrder));
                int updateEtOrder = etOrderService.updateEtOrder(etOrder);
                if(updateEtOrder>0){
                    logger.info("计算订单距离成功：【orderNo="+etOrder.getOrderNo()+"】");
                }
            }
        }
        return AjaxResult.success();
    }

    /**
     * 查询版本并更新
     */
    @GetMapping("/queryVersion")
    public AjaxResult queryVersion(String sn)
    {
        if(StrUtil.isNotBlank(sn)){
            return AjaxResult.success(asDeviceService.updateVersion(sn));
        }
        List<AsDevice> asDevices = asDeviceService.selectAsDeviceList(new AsDevice());
        for (AsDevice asDevice:asDevices) {
            asDeviceService.updateVersion(asDevice.getSn());
        }
        return AjaxResult.success();
    }
    @GetMapping("/sendMsg")
    public AjaxResult getUserInfo(String orderNo)
    {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        /** 发送一个短信给运营商*/
        SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
        logger.info("【微信支付回调】还车拍照审核，获取到运营商：【{}】", JSON.toJSON(sysDept));
        if(ObjectUtil.isNull(sysDept.getAppUserId())){
            throw new ServiceException("该运营商【"+sysDept.getDeptName()+"】未绑定微信用户");
        }
        AsUser asUser1 = asUserMapper.selectUserById(sysDept.getAppUserId());
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("name",order.getPayFee());
        String phone = asUser1.getPhonenumber();
        SendSmsVo sendSmsVo = new SendSmsVo();
        sendSmsVo.setMobile(phone);
        sendSmsVo.setTemplateCode(templateCode);
        sendSmsVo.setParam(jsonObject.toJSONString());
        sendSmsVo.setSignName(signName);
        SendSmsResponse response = null;
        logger.info("【微信支付回调】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
        try {
            response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
        } catch (ClientException e) {
            e.printStackTrace();
        }
        logger.info("【微信支付回调】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));
        return AjaxResult.success();
    }

    @GetMapping("/sendMsg2")
    public AjaxResult getUserInfo2(String orderNo)
    {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        /** 发送一个短信给运营商*/
        SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
        logger.info("【用户付款通知】用户付款通知，获取到运营商：【{}】",JSON.toJSON(sysDept));
        if(ObjectUtil.isNull(sysDept.getAppUserId())){
            throw new ServiceException("该运营商【"+sysDept.getDeptName()+"】未绑定微信用户");
        }
        AsUser asUser1 = asUserMapper.selectUserById(sysDept.getAppUserId());
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(order.getSn());

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("order", order.getOrderNo());
        jsonObject.put("cheno", "");
        jsonObject.put("snno", order.getSn());
        jsonObject.put("fee", order.getPayFee());

        String phone = asUser1.getPhonenumber();
        SendSmsVo sendSmsVo = new SendSmsVo();
        sendSmsVo.setMobile(phone);
        sendSmsVo.setTemplateCode(templateCode2);
        sendSmsVo.setParam(jsonObject.toJSONString());
        sendSmsVo.setSignName(signName);
        SendSmsResponse response = null;
        logger.info("【用户付款通知】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
        try {
            response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
        } catch (ClientException e) {
            e.printStackTrace();
        }
        logger.info("【用户付款通知】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));
        return AjaxResult.success();
    }

    @GetMapping("/sendMsg3")
    public AjaxResult getUserInfo3(Long deptId)
    {
        SysDept sysDept = deptService.selectDeptById(deptId);
        String deptName = sysDept.getDeptName();
        /** 提现申请发送一个短信给丁建帮*/
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("name", deptName);

        SendSmsVo sendSmsVo = new SendSmsVo();
        sendSmsVo.setMobile(phone);
        sendSmsVo.setTemplateCode(templateCode3);
        sendSmsVo.setParam(jsonObject.toJSONString());
        sendSmsVo.setSignName(signName);
        SendSmsResponse response = null;
        logger.info("【提现申请】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
        try {
            response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
        } catch (ClientException e) {
            e.printStackTrace();
        }
        logger.info("【提现申请】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));

        return AjaxResult.success();
    }

    /**
     * app忘记密码
     */
    @Log(title = "app忘记密码", businessType = BusinessType.UPDATE)
    @PutMapping("/forgetAppPwd")
    public AjaxResult forgetAppPwd(String phone,String phoneCode, String newPassword,String uuid)
    {
        logger.info("【app忘记密码】请求参数：phone:{},phoneCode:{},newPassword:{}",phone,phoneCode,newPassword);
        return success(loginService.forgetAppPwd(phone,phoneCode,newPassword,uuid));
    }

    /**
     * 查询硬件版本列表
     */
    @GetMapping("/hardwareVersion/list")
    public AjaxResult list(EtHardwareVersion etHardwareVersion)
    {
        List<EtHardwareVersion> list = etHardwareVersionService.selectEtHardwareVersionList(etHardwareVersion);
        return success(list);
    }

    /**
     * 获取token
     */
    @GetMapping("/getToken")
    public AjaxResult getToken()
    {
        String token = null;
        try {
            token = Token.getToken();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        }
        return success(token);
    }

    /**
     * 修复押金抵扣资金流水
     */
    @GetMapping(value = "/repair/deduction")
    public AjaxResult repairDeduction(String startDateStr,String endDateStr,Long areaId)
    {
        logger.info("【修复押金抵扣请求】：区域id={}",areaId);
        return toAjax(etOrderService.repairDeduction(startDateStr,endDateStr,areaId));
    }

    /**
     * 定位转换
     */
    @GetMapping(value = "/position/conversion")
    public AjaxResult repairDeduction(BigDecimal lon,BigDecimal lat)
    {
        logger.info("WGS84经纬度（未计算）：" + lon + "---" + lat);
        // 除以100
        lon = lon.divide(new BigDecimal(100), 10, RoundingMode.HALF_UP);
        lat = lat.divide(new BigDecimal(100), 10, RoundingMode.HALF_UP);
        logger.info("WGS84经纬度（除以100后）：" + lon + "---" + lat);
        // 取出lon中后面的小数点
        String[] lonStr = getDecimalPart(lon);
        String[] latStr = getDecimalPart(lat);
        logger.info("WGS84经纬度（截取小数点）：" + lonStr[0] + "---" + lonStr[1] + "---"+ latStr[0]+"---"+ latStr[1]);
        // 再将结果乘以5/3
        String lon2 = "0."+ lonStr[1];
        String lat2 = "0."+ latStr[1];
        BigDecimal lons = new BigDecimal(lon2).multiply(new BigDecimal(5).divide(new BigDecimal(3), 8, RoundingMode.HALF_UP));
        BigDecimal lats = new BigDecimal(lat2).multiply(new BigDecimal(5).divide(new BigDecimal(3), 8, RoundingMode.HALF_UP));
        BigDecimal lo = new BigDecimal(lonStr[0]).add(lons);
        BigDecimal la = new BigDecimal(latStr[0]).add(lats);
        logger.info("WGS84经纬度（计算后）：" + lo + "---" + la);
        lo = lo.setScale(8, RoundingMode.HALF_UP);
        la = la.setScale(8, RoundingMode.HALF_UP);
        logger.info("WGS84经纬度（保留8为小数）：" + lo + "---" + la);
        double[] doubles = GpsCoordinateUtils.calWGS84toGCJ02(la.doubleValue(), lo.doubleValue());
        lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
        lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
        logger.info("转换后的GCJ02经纬度：" + lon + "---" + lat);
        return success(lon+","+lat);
    }

    private static String[] getDecimalPart(BigDecimal number) {
        // 将BigDecimal转换为字符串
        String numberStr = number.toPlainString();

        // 找到小数点的位置
        int indexOfDecimal = numberStr.indexOf(".");

        // 初始化结果数组
        String[] parts = new String[2];

        // 如果有小数点
        if (indexOfDecimal >= 0) {
            parts[0] = numberStr.substring(0, indexOfDecimal); // 整数部分
            parts[1] = numberStr.substring(indexOfDecimal + 1); // 小数部分
        } else {
            // 如果没有小数点，整数部分为整个字符串，小数部分为空
            parts[0] = numberStr;
            parts[1] = "";
        }

        return parts;
    }

    /**
     * 查询优惠券列表
     */
    @GetMapping("/couponList")
    public AjaxResult list(EtCoupon etCoupon)
    {
        if(etCoupon.getType().contains(",")){
            String[] typeList = etCoupon.getType().split(",");
            etCoupon.setTypeList(typeList);
            etCoupon.setType(null);
        }
        List<EtCoupon> list = etCouponService.selectEtCouponList(etCoupon);
        return success(list);
    }

    /**
     * 不同状态的订单
     */
    @GetMapping("/getErrorOrder")
    public AjaxResult getErrorOrder()
    {
        List<EtOrder> errorOrders = new ArrayList<>();
        // 当前有骑行中的订单
        List<EtOrder> orders = etOrderService.getCurrentOrderList();
        for (EtOrder order:orders) {
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            if(ObjectUtil.isNotNull(device) && device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NORMAL)){
                errorOrders.add(order);
            }
        }
        return success(errorOrders);
    }

    /**
     * 不同状态的车辆
     */
    @GetMapping("/getErrorDevice")
    public AjaxResult getErrorDevice()
    {
        List<AsDevice> errorDevices = new ArrayList<>();
        // 2. 如果车辆状态是骑行中或临时锁车，查询当前没有订单，则修改车辆状态为待骑行
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.in("status", "3","4"); // 设备状态正常
        // 查询所有设备
        List<AsDevice> allDevices = asDeviceMapper.selectList(wrapper);
        for(AsDevice device:allDevices){
            if(ObjectUtil.isNull(etOrderService.getCurrentOrder2(device.getSn()))){
                errorDevices.add(device);
            }
        }
        return success(errorDevices);
    }

    /**
     * 押金抵扣不成功订单
     */
    @GetMapping("/getDeductionErrorOrder")
    public AjaxResult getDeductionErrorOrder()
    {
        List<EtOrder> orders = etOrderMapper.deductionErrorOrderList();
        return success(orders);
    }

}
