package com.ruoyi.web.controller.system;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Channel;
import com.ruoyi.system.domain.ChannelQuery;
import com.ruoyi.system.domain.ChannelVO;
import com.ruoyi.system.service.EtChannelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 充值渠道Controller
 *
 * @author ruoyi
 * @date 2024-04-15
 */
@RestController
@RequestMapping("/system/channel")
public class EtChannelController extends BaseController
{
    @Autowired
    private EtChannelService smEtChannelService;

    /**
     * 查询充值渠道列表
     */
    @PreAuthorize("@ss.hasPermi('system:channel:list')")
    @GetMapping("/list")
    public TableDataInfo list(ChannelQuery smChannel)
    {
        startPage();
        List<ChannelVO> list = smEtChannelService.selectSmChannelList(smChannel);
        return getDataTable(list);
    }

    /**
     * 导出充值渠道列表
     */
    @PreAuthorize("@ss.hasPermi('system:channel:export')")
    @Log(title = "充值渠道", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ChannelQuery smChannel)
    {
        List<ChannelVO> list = smEtChannelService.selectSmChannelList(smChannel);
        ExcelUtil<ChannelVO> util = new ExcelUtil<ChannelVO>(ChannelVO.class);
        util.exportExcel(response, list, "充值渠道数据");
    }

    /**
     * 获取充值渠道详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:channel:query')")
    @GetMapping(value = "/{channelId}")
    public AjaxResult getInfo(@PathVariable("channelId") Long channelId)
    {
        return success(smEtChannelService.selectSmChannelByChannelId(channelId));
    }

    /**
     * 修改充值渠道
     */
    @PreAuthorize("@ss.hasPermi('system:channel:edit')")
    @Log(title = "充值渠道", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) Channel form)
    {
        Channel channel = new Channel();
        channel.setChannelId(form.getChannelId());
        channel.setEnabled(form.getEnabled());
        channel.setCostRate(form.getCostRate());
        channel.setPicture(form.getPicture());
        channel.setMerchantId(form.getMerchantId());
        channel.setApiV3Key(form.getApiV3Key());
        channel.setNotifyUrl(form.getNotifyUrl());
        channel.setPrivateKeyPath(form.getPrivateKeyPath());
        channel.setMerchantSerialNumber(form.getMerchantSerialNumber());
        channel.setRefundNotifyUrl(form.getRefundNotifyUrl());
        return toAjax(smEtChannelService.updateSmChannel(channel));
    }

}
