package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtLocationLog;
import com.ruoyi.system.service.IEtLocationLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 定位日志Controller
 *
 * @author ruoyi
 * @date 2024-09-12
 */
@RestController
@RequestMapping("/system/locationLog")
public class EtLocationLogController extends BaseController
{
    @Autowired
    private IEtLocationLogService etLocationLogService;

    /**
     * 查询定位日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:locationLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtLocationLog etLocationLog)
    {
        startPage();
        List<EtLocationLog> list = etLocationLogService.selectEtLocationLogList(etLocationLog);
        etLocationLogService.analytic(list);
        return getDataTable(list);
    }

    /**
     * 导出定位日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:locationLog:export')")
    @Log(title = "定位日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtLocationLog etLocationLog)
    {
        List<EtLocationLog> list = etLocationLogService.selectEtLocationLogList(etLocationLog);
        ExcelUtil<EtLocationLog> util = new ExcelUtil<EtLocationLog>(EtLocationLog.class);
        util.exportExcel(response, list, "定位日志数据");
    }

    /**
     * 获取定位日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:locationLog:query')")
    @GetMapping(value = "/{locationId}")
    public AjaxResult getInfo(@PathVariable("locationId") Long locationId)
    {
        return success(etLocationLogService.selectEtLocationLogByLocationId(locationId));
    }

    /**
     * 新增定位日志
     */
    @PreAuthorize("@ss.hasPermi('system:locationLog:add')")
    @Log(title = "定位日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtLocationLog etLocationLog)
    {
        return toAjax(etLocationLogService.insertEtLocationLog(etLocationLog));
    }

    /**
     * 修改定位日志
     */
    @PreAuthorize("@ss.hasPermi('system:locationLog:edit')")
    @Log(title = "定位日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtLocationLog etLocationLog)
    {
        return toAjax(etLocationLogService.updateEtLocationLog(etLocationLog));
    }

    /**
     * 删除定位日志
     */
    @PreAuthorize("@ss.hasPermi('system:locationLog:remove')")
    @Log(title = "定位日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{locationIds}")
    public AjaxResult remove(@PathVariable Long[] locationIds)
    {
        return toAjax(etLocationLogService.deleteEtLocationLogByLocationIds(locationIds));
    }
}
