package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.system.service.IAsDeviceService;
import com.ruoyi.system.service.IEtFeeRuleService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtModel;
import com.ruoyi.system.service.IEtModelService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 车辆型号Controller
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@RestController
@RequestMapping("/system/model")
public class EtModelController extends BaseController
{
    @Autowired
    private IEtModelService etModelService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;


    /**
     * 查询车辆型号列表
     */
    @PreAuthorize("@ss.hasPermi('system:model:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtModel etModel)
    {
        startPage();
        List<EtModel> list = etModelService.selectEtModelList(etModel);
        return getDataTable(list);
    }

    /**
     * 导出车辆型号列表
     */
    @PreAuthorize("@ss.hasPermi('system:model:export')")
    @Log(title = "车辆型号", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtModel etModel)
    {
        List<EtModel> list = etModelService.selectEtModelList(etModel);
        ExcelUtil<EtModel> util = new ExcelUtil<EtModel>(EtModel.class);
        util.exportExcel(response, list, "车辆型号数据");
    }

    /**
     * 获取车辆型号详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:model:query')")
    @GetMapping(value = "/{modelId}")
    public AjaxResult getInfo(@PathVariable("modelId") Long modelId)
    {
        AjaxResult ajax = AjaxResult.success();
        EtModel model = etModelService.selectEtModelByModelId(modelId);
        ajax.put(AjaxResult.DATA_TAG, model);
        ajax.put("ruleIds", etFeeRuleService.selectRuleListByModelId(modelId));
        return ajax;
    }

    /**
     * 新增车辆型号
     */
    @PreAuthorize("@ss.hasPermi('system:model:add')")
    @Log(title = "车辆型号", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtModel etModel)
    {
        return toAjax(etModelService.insertEtModel(etModel));
    }

    /**
     * 修改车辆型号
     */
    @PreAuthorize("@ss.hasPermi('system:model:edit')")
    @Log(title = "车辆型号", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtModel etModel)
    {
        return toAjax(etModelService.updateEtModel(etModel));
    }

    /**
     * 删除车辆型号
     */
    @PreAuthorize("@ss.hasPermi('system:model:remove')")
    @Log(title = "车辆型号", businessType = BusinessType.DELETE)
	@DeleteMapping("/{modelIds}")
    public AjaxResult remove(@PathVariable Long[] modelIds)
    {
        for (Long modelId:modelIds) {
            // 判断该型号下是否有存在车辆
            if(asDeviceService.selectCountByModelId(modelId) > 0){
                return AjaxResult.warn("该型号【"+modelId+"】下存在车辆，无法删除！");
            }
        }
        return toAjax(etModelService.deleteEtModelByModelIds(modelIds));
    }
}
