package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtReconciliation;
import com.ruoyi.system.service.IEtReconciliationService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 平台对账Controller
 *
 * @author qzz
 * @date 2024-10-03
 */
@RestController
@RequestMapping("/system/reconciliation2")
public class EtReconciliationController extends BaseController
{
    @Autowired
    private IEtReconciliationService etReconciliationService;

    /**
     * 查询平台对账列表
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation2:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtReconciliation etReconciliation)
    {
        startPage();
        List<EtReconciliation> list = etReconciliationService.selectEtReconciliationList(etReconciliation);
        return getDataTable(list);
    }

    /**
     * 导出平台对账列表
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation2:export')")
    @Log(title = "平台对账", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtReconciliation etReconciliation)
    {
        List<EtReconciliation> list = etReconciliationService.selectEtReconciliationList(etReconciliation);
        ExcelUtil<EtReconciliation> util = new ExcelUtil<EtReconciliation>(EtReconciliation.class);
        util.exportExcel(response, list, "平台对账数据");
    }

    /**
     * 获取平台对账详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation2:query')")
    @GetMapping(value = "/{reconciliationId}")
    public AjaxResult getInfo(@PathVariable("reconciliationId") Long reconciliationId)
    {
        return success(etReconciliationService.selectEtReconciliationByReconciliationId(reconciliationId));
    }

    /**
     * 新增平台对账
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation2:add')")
    @Log(title = "平台对账", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtReconciliation etReconciliation)
    {
        return toAjax(etReconciliationService.insertEtReconciliation(etReconciliation));
    }

    /**
     * 修改平台对账
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation2:edit')")
    @Log(title = "平台对账", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtReconciliation etReconciliation)
    {
        return toAjax(etReconciliationService.updateEtReconciliation(etReconciliation));
    }

    /**
     * 删除平台对账
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation2:remove')")
    @Log(title = "平台对账", businessType = BusinessType.DELETE)
	@DeleteMapping("/{reconciliationIds}")
    public AjaxResult remove(@PathVariable Long[] reconciliationIds)
    {
        return toAjax(etReconciliationService.deleteEtReconciliationByReconciliationIds(reconciliationIds));
    }
}
