package com.ruoyi.web.controller.system;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.IEtCouponClaimLogService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysMenuService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Set;

/**
 * 登录验证
 *
 * @author ruoyi
 */
@Slf4j
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private IAsUserService userService;

    @Autowired
    private ISysDeptService deptService;

    @Resource
    private AsUserMapper asUserMapper;

    @Autowired
    private IEtCouponClaimLogService etCouponClaimLogService;


    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody)
    {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo()
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }

    /**
     * 获取app用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getAppInfo")
    public AjaxResult getAppInfo(Long userId)
    {
        AjaxResult ajax = AjaxResult.success();
        AsUser user;
        if(ObjectUtil.isNotNull(userId)){
            user =  userService.selectUserById(userId);
        }else{
            user = SecurityUtils.getLoginUser().getAsUser();
            log.info("获取到当前登录用户:{}", JSON.toJSON(user));
            user = asUserMapper.selectUserById(user.getUserId());
        }
        ajax.put("user", user);
        return ajax;
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        SysDept sysDept = deptService.selectDeptById(SecurityUtils.getDeptId());
//        if(!sysDept.getIsProfitSharing().equals("true")){
//            // 去掉 menus 中的名字为‘合伙人管理’的菜单
//            menus.removeIf(menu -> menu.getMenuName().equals("合伙人管理"));
//        }
        if(sysDept.getDeptId() == 101){
            // 找出 menus 中的名字为‘财务管理’的菜单，获取到它子菜单
            menus.stream().filter(menu -> menu.getMenuName().equals("财务管理")).forEach(menu -> {
                List<SysMenu> children = menu.getChildren();
                children.removeIf(child -> child.getMenuName().equals("提现记录"));
            });
        }
        return AjaxResult.success(menuService.buildMenus(menus));
    }

    @PostMapping("/appCodeLogin")
    public AjaxResult appCodeLogin(@RequestBody LoginBody loginBody) {
        AjaxResult ajax = AjaxResult.success();
        /**通过手机号找到用户名*/
        String phone = loginBody.getPhone();
        // 生成令牌
        String token = loginService.appCodeLogin(phone, loginBody.getPhoneCode(),loginBody.getPassword(), loginBody.getUuid(), loginBody.getAreaId(), loginBody.getOpenid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 微信登录
     */
    @PostMapping("/wxlogin")
    @Log(title = "微信登录", businessType = BusinessType.LOGIN)
    public AjaxResult wxlogin(String mobileCode,String jsCode,String areaId) {
        if(StrUtil.isBlank(areaId)){
            log.info("没有传入areaId参数：【areaId={}】",areaId);
            return AjaxResult.error("请传areaId参数"+"【areaId="+areaId+"】");
        }
        log.info("【微信登录/wxlogin】areaId参数：【areaId={}】",areaId);
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.wxloing(mobileCode,jsCode,areaId);
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 根据openid静默登录
     */
    @PostMapping("/loginByopenid")
    public AjaxResult loginByopenid(String jsCode,String areaId) {
        if(StrUtil.isBlank(areaId)){
            log.info("没有传入areaId参数：【areaId={}】",areaId);
            return new AjaxResult(HttpStatus.NOT_IMPLEMENTED, "请传areaId参数"+"【areaId="+areaId+"】", null);
        }
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.loginByopenid(jsCode,areaId);
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }
}
