package com.ruoyi.system.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.domain.vo.AsDeviceVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 设备Mapper接口
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
public interface AsDeviceMapper extends BaseMapper<AsDevice>
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public AsDevice selectAsDeviceByDeviceId(Long deviceId);

    /**
     * 根据mac号查询设备信息
     *
     * @param mac 设备列表主键
     * @return 设备列表
     */
    public AsDevice selectAsDeviceByMac(String mac);

    /**
     * 根据SN查询设备信息
     *
     * @param sn 设备SN
     * @return 设备列表
     */
    public AsDevice selectAsDeviceBySn(String sn);


    /**
     * 查询设备列表
     *
     * @param asDevice 设备
     * @return 设备集合
     */
    public List<AsDevice> selectAsDeviceList(AsDevice asDevice);

    /**
     * 查询设备列表（带数据隔离）
     *
     * @param asDevice 设备
     * @return 设备集合
     */
    public List<AsDevice> selectAsDeviceListWithIsolate(AsDevice asDevice);

    /**
     * 新增设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int insertAsDevice(AsDevice asDevice);

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
//    @Log(title = "修改设备", businessType = BusinessType.UPDATE)
    public int updateAsDevice(AsDevice asDevice);

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int updateAsDevice2(AsDevice asDevice);

    /**
     * 根据SN修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int updateAsDeviceBySn(AsDevice asDevice);

    /**
     * 根据MAC修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    public int updateAsDeviceByMac(AsDevice asDevice);

    /**
     * 删除设备
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    public int deleteAsDeviceByDeviceId(Long deviceId);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAsDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 一键上线
     *
     * @param deviceIds 需要一键上线的设备主键集合
     * @return 结果
     */
    public int oneClickOnline(Long[] deviceIds);

    /**
     * 一键下线
     *
     * @param deviceIds 需要一键下线的设备主键集合
     * @return 结果
     */
    public int oneClickOffline(Long[] deviceIds);

    /**
     * 一键入仓
     *
     * @param deviceIds 需要一键入仓的设备主键集合
     * @return 结果
     */
    public int oneClickWarehousing(Long[] deviceIds);


    /**
     * 有订单车辆
     */
    String getInOrderDevice(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 无订单车辆
     */
    String getNoOrderDevice(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

//    public int batchDisable(String[] sns);


    AsDevice checkSNUnique(String sn);

    AsDevice checkMACUnique(String mac);

    /**
     * 已绑定版本数
     */
    int countBoundNum(Long id);

    /**
     * 未绑定版本数
     */
    int countUnBoundNum(Long id);

    /**
     * 未绑定版本数
     */
    List<AsDeviceVO> selectAllDevice();
}
