package com.ruoyi.system.mapper;

import java.math.BigDecimal;
import java.util.List;
import com.ruoyi.system.domain.EtCapitalFlow;
import org.apache.ibatis.annotations.Param;

/**
 * 资金流水Mapper接口
 *
 * @author 邱贞招
 * @date 2024-05-24
 */
public interface EtCapitalFlowMapper
{
    /**
     * 查询资金流水
     *
     * @param flowId 资金流水主键
     * @return 资金流水
     */
    public EtCapitalFlow selectEtCapitalFlowByFlowId(Long flowId);

    /**
     * 查询资金流水列表
     *
     * @param etCapitalFlow 资金流水
     * @return 资金流水集合
     */
    public List<EtCapitalFlow> selectEtCapitalFlowList(EtCapitalFlow etCapitalFlow);

    /**
     * 新增资金流水
     *
     * @param etCapitalFlow 资金流水
     * @return 结果
     */
    public int insertEtCapitalFlow(EtCapitalFlow etCapitalFlow);

    /**
     * 修改资金流水
     *
     * @param etCapitalFlow 资金流水
     * @return 结果
     */
    public int updateEtCapitalFlow(EtCapitalFlow etCapitalFlow);

    /**
     * 删除资金流水
     *
     * @param flowId 资金流水主键
     * @return 结果
     */
    public int deleteEtCapitalFlowByFlowId(Long flowId);

    /**
     * 批量删除资金流水
     *
     * @param flowIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEtCapitalFlowByFlowIds(Long[] flowIds);

    EtCapitalFlow selectEtCapitalFlowByOutTradeNo(String outTradeNo);

    List<EtCapitalFlow> selectEtCapitalFlowByOrderNo(String orderNo);

    /**
     * 已提现金额
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getWithdrawnFee(@Param("areaId") Long areaId, @Param("status") String status);

    /**
     * 手续费
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getHandlingFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd,  @Param("sn") String sn, @Param("areaId") Long areaId);

    /**
     * 手续费
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getHandlingFee2(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd,  @Param("sn") String sn, @Param("areaId") Long areaId,@Param("channelId") Long channelId);

    /**
     * 平台服务费
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getServiceFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd,  @Param("sn") String sn, @Param("areaId") Long areaId);
    /**
     * 平台服务费
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getServiceFee2(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd,  @Param("sn") String sn, @Param("areaId") Long areaId,@Param("channelId") Long channelId);

    /**
     * 骑行订单收入
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getTotalAmount(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd,  @Param("sn") String sn, @Param("areaId") Long areaId);

    /**
     * 押金抵扣收入
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getDeductionAmount(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd,  @Param("sn") String sn, @Param("areaId") Long areaId);

    /**
     * 已支付骑行订单
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getOrderPaidAmount(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") Long areaId,@Param("channelId") Long channelId);

    /**
     * 统计所有用户今日账变
     *
     * @param areaId 运营区id
     * @return
     */
    BigDecimal getAllUserReceipts(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") Long areaId,@Param("channelId") Long channelId);
}
