package com.ruoyi.system.mapper;

import java.math.BigDecimal;
import java.util.List;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.EtOrderQuery;
import org.apache.ibatis.annotations.Param;

/**
 * 订单Mapper接口
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
public interface EtOrderMapper
{
    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    public EtOrder selectEtOrderByOrderId(Long orderId);

    /**
     * 根据订单号查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    public EtOrder selectEtOrderByOrderNo(String orderNo);

    /**
     * 根据微信内部订单号查询订单
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    public EtOrder selectEtOrderByOutTradeNo(String outTradeNo);

    /**
     * 查询订单列表
     *
     * @param etOrder 订单
     * @return 订单集合
     */
    public List<EtOrder> selectEtOrderList(EtOrder etOrder);

    /**
     * 查询订单列表（不带路由）
     *
     * @param etOrder 订单
     * @return 订单集合
     */
    public List<EtOrder> selectEtOrderListNoRoute(EtOrder etOrder);

    /**
     * 查询订单列表（不带路由）
     *
     * @param etOrder 订单
     * @return 订单集合
     */
    public List<EtOrder> selectEtOrderListNoRoute2(EtOrderQuery etOrder);

    /**
     * 新增订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    public int insertEtOrder(EtOrder etOrder);

    /**
     * 修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    public int updateEtOrder(EtOrder etOrder);

    /**
     * 根据订单号修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    public int updateEtOrderByOrderNo(EtOrder etOrder);

    /**
     * 删除订单
     *
     * @param orderId 订单主键
     * @return 结果
     */
    public int deleteEtOrderByOrderId(Long orderId);

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEtOrderByOrderIds(Long[] orderIds);

    /**
     * 查询当前用户是否有正在进行中的订单
     *
     * @param userId 用户id
     * @param orderNo 排除当前订单号
     * @return 结果
     */
    public List<EtOrder> isInOrder(@Param("userId") Long userId,@Param("orderNo") String orderNo);

    /**
     * 查询当前用户是否有正在进行中的审核订单
     *
     * @param userId 用户id
     * @param orderNo 排除当前订单号
     * @return 结果
     */
    public List<EtOrder> isInAuditOrder(@Param("userId") Long userId,@Param("orderNo") String orderNo);

    /**
     * 查询当前车辆是否有正在进行中的订单
     *
     * @param sn 用户id
     * @return 结果
     */
    public int isInOrderBySn(@Param("sn") String sn);

    /**
     * 检验预约订单只能有一个
     *
     * @param userId 用户id
     * @return 结果
     */
    int checkIsUnique(Long userId);

    /**
     * 总收入
     */
    String getTotalIncome(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 累计待支付
     */
    String getTotalUnpaid(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 已支付
     */
    String getTotalPaid(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 已退款
     */
    String getTotalRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 骑行已支付
     */
    String getTotalRidingFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 骑行已退款
     */
    String getTotalRidingRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 调度费已支付
     */
    String getTotalDispatchFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 调度费已退款
     */
    String getTotalDispatchRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 预约费已支付
     */
    String getTotalAppointmentFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 预约费已退款
     */
    String getTotalAppointmentRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 停车点外调度费已支付
     */
    String getTotalManageFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 停车点外调度费已退款
     */
    String getTotalManageRefund(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);


    /**
     * 已支付订单
     */
    String getPaidOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 骑行中订单
     */
    String getRidingOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 退款订单
     */
    String getRefundOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 待支付订单
     */
    String getUnpaidOrder(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 还车待审核订单数
     */
    int getAuditOrderNum(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 待审核还车押金扣款
     */
    BigDecimal getReturnOrderDeductFee(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") String areaId);

    /**
     * 获取订单数
     */
    int getOrderNum(@Param("timeStart") String timeStart, @Param("timeEnd") String timeEnd, @Param("areaId") Long areaId);

    /**
     * 订单总金额
     */
    BigDecimal getOrderFee(@Param("startDateStr") String timeStart, @Param("endDateStr") String timeEnd, @Param("areaId") Long areaId);

    /**
     * 手续费
     */
    BigDecimal getServiceFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr);

    /**
     * 支付金额
     */
    BigDecimal getPayFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn, @Param("areaId") Long areaId);

    /**
     * 已退款
     */
    BigDecimal getRefundFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn, @Param("areaId") Long areaId);

    /**
     * 已退款
     */
    BigDecimal getRefundFee2(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn, @Param("areaId") Long areaId,@Param("channelId") Long channelId);

    /**
     * 押金退款
     */
    BigDecimal getDepositRefundFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn, @Param("areaId") Long areaId,@Param("channelId") Long channelId);

    /**
     * 收入
     */
    BigDecimal getIncome(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("sn") String sn, @Param("areaId") Long areaId);

    /**
     * 最近一笔订单
     */
    EtOrder selectLatestOrder(EtOrder etOrder);

    /**
     * 最近一笔订单（包含押金订单）
     */
    EtOrder selectLatestOrder2(EtOrder etOrder);

    /**
     * 最近一笔订单
     */
    List<EtOrder> selectLatestOrderList(EtOrder etOrder);

    /**
     * 预约未完成订单处理
     */
    List<EtOrder> selectAppointmentUnfinished();

    /**
     * 押金未退还订单处理
     */
    List<EtOrder> selectUnrefundableDepositOrder();

    /**
     * 已完成订单的用户列表
     */
    List<EtOrder> selectUserListFinishOrder();

    /**
     * 待结算订单
     */
    List<EtOrder> selectNeedDividendOrder();

    /**
     * 根据订单号删除订单
     *
     * @param orderNo 订单号
     * @return 结果
     */
    int deleteEtOrderByOrderNo(String orderNo);

    /**
     * 合伙人分账
     */
    BigDecimal getPartnerFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("areaId") Long areaId);

    /**
     * 押金订单收入
     */
    BigDecimal getDepositAmount(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("areaId") Long areaId,@Param("channelId") Long channelId);

//    /**
//     * 平台服务费
//     */
//    BigDecimal getPlatformServiceFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr, @Param("areaId") Long areaId);


//    Integer getAppCount();

    List<EtOrder> selectDeductionList(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr,@Param("areaId") Long areaId);

//    /**
//     * 判断当前是否有正在骑行中的订单
//     */
//    int getCurrentOrderNum(String sn);

    /**
     * 7天前未支付的订单
     */
    List<EtOrder> selectToBePaidEtOrderList();

    /**
     * 查询所有正在骑行中的订单
     */
    List<EtOrder> getCurrentOrderList();

    /**
     * 押金抵扣不成功的修复
     */
    List<EtOrder> deductionErrorOrderList();

    /**
     * 平台对账--总支付流水（包含订单+押金）
     */
    BigDecimal getTotalPaidFee(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr ,@Param("areaId") Long areaId,@Param("channelId") Long channelId);

    /**
     * 平台对账--押金抵扣金额
     */
    BigDecimal getDepositDeductionAmount(@Param("startDateStr") String startDateStr , @Param("endDateStr") String endDateStr ,@Param("areaId") Long areaId,@Param("channelId") Long channelId);

}
