package com.ruoyi.system.mapper;

import java.util.List;

import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.EtOrderQuery;
import com.ruoyi.system.domain.EtRefund;

/**
 * 退款订单Mapper接口
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
public interface EtRefundMapper
{
    /**
     * 查询退款订单
     *
     * @param id 退款订单主键
     * @return 退款订单
     */
    public EtRefund selectEtRefundById(Long id);

    /**
     * 查询退款订单列表
     *
     * @param etRefund 退款订单
     * @return 退款订单集合
     */
    public List<EtRefund> selectEtRefundList(EtRefund etRefund);

    /**
     * 查询退款订单列表
     *
     * @param orderQuery 退款订单
     * @return 退款订单集合
     */
    public List<EtRefund> selectEtRefundList2(EtOrderQuery orderQuery);

    /**
     * 新增退款订单
     *
     * @param etRefund 退款订单
     * @return 结果
     */
    public int insertEtRefund(EtRefund etRefund);

    /**
     * 修改退款订单
     *
     * @param etRefund 退款订单
     * @return 结果
     */
    public int updateEtRefund(EtRefund etRefund);

    /**
     * 删除退款订单
     *
     * @param id 退款订单主键
     * @return 结果
     */
    public int deleteEtRefundById(Long id);

    /**
     * 批量删除退款订单
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEtRefundByIds(Long[] ids);

    /**
     * 根据退款单号修改退款订单
     *
     * @param etRefund 退款订单
     * @return 结果
     */
    public int updateEtRefundByRefundNo(EtRefund etRefund);

    /**
     * 根据订单号查询退款订单对象
     *
     * @param orderNo 订单号
     * @return 退款订单
     */
    List<EtRefund> selectEtRefundByOrderNo(String orderNo);

    /**
     * 根据订单号查询退款订单对象
     *
     * @param refundNo 退款订号
     * @return 退款订单
     */
    EtRefund selectEtRefundByRefundNo(String refundNo);

    /**
     * 判断是否用户是否还有未退款的押金
     *
     * @param userId 用户id
     * @return 结果
     */
    List<EtOrder> checkUserDeposit(Long userId);
}
