package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.system.domain.Channel;
import com.ruoyi.system.domain.ChannelQuery;
import com.ruoyi.system.domain.ChannelVO;
import com.ruoyi.system.mapper.EtChannelMapper;
import com.ruoyi.system.service.EtChannelService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 充值渠道Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-15
 */
@Service
public class EtChannelServiceImpl implements EtChannelService
{
    @Resource
    private EtChannelMapper etChannelMapper;

    /**
     * 查询充值渠道
     *
     * @param channelId 充值渠道主键
     * @return 充值渠道
     */
    @Override
    public ChannelVO selectSmChannelByChannelId(Long channelId)
    {
        return etChannelMapper.selectSmChannelByChannelId(channelId);
    }

    /**
     * 查询充值渠道列表
     *
     * @param smChannel 充值渠道
     * @return 充值渠道
     */
    @Override
    public List<ChannelVO> selectSmChannelList(ChannelQuery smChannel)
    {
        return etChannelMapper.selectSmChannelList(smChannel);
    }

    /**
     * 新增充值渠道
     *
     * @param channel 充值渠道
     * @return 结果
     */
    @Override
    public int insertSmChannel(Channel channel)
    {
        return etChannelMapper.insertSmChannel(channel);
    }

    /**
     * 修改充值渠道
     *
     * @param channel 充值渠道
     * @return 结果
     */
    @Override
    public int updateSmChannel(Channel channel)
    {
        return etChannelMapper.updateSmChannel(channel);
    }

    /**
     * 批量删除充值渠道
     *
     * @param channelIds 需要删除的充值渠道主键
     * @return 结果
     */
    @Override
    public int deleteSmChannelByChannelIds(Long[] channelIds)
    {
        return etChannelMapper.deleteSmChannelByChannelIds(channelIds);
    }

    /**
     * 删除充值渠道信息
     *
     * @param channelId 充值渠道主键
     * @return 结果
     */
    @Override
    public int deleteSmChannelByChannelId(Long channelId)
    {
        return etChannelMapper.deleteSmChannelByChannelId(channelId);
    }

    @Override
    public List<ChannelVO> selectEnabledRechargeList() {
        ChannelQuery dto = new ChannelQuery();
        dto.setEnabled(true);
        return this.selectSmChannelList(dto);
    }

    /**
     * 查询充值渠道映射表
     *
     * @param query     查询条件
     * @param keyMapper 映射函数
     */
    @Override
    public <K> Map<K, ChannelVO> selectMap(ChannelQuery query, Function<? super Channel, ? extends K> keyMapper) {
        List<ChannelVO> list = this.selectSmChannelList(query);
//        if (CollectionUtils.isEmptyElement(list)) {
//            return Collections.emptyMap();
//        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }

    /**
     * 根据证书序列号获取支付渠道
     *
     * @param wechatpaySerial 证书序列号
     * @return 结果
     */
    @Override
    public ChannelVO selectChannelBySerialNumber(String wechatpaySerial) {
        ChannelVO channel = etChannelMapper.selectChannelBySerialNumber(wechatpaySerial);
        return channel;
    }
}
