package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.onenet.ResponseVo;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.mapper.EtModelMapper;
import com.ruoyi.system.mapper.EtModelRuleMapper;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.service.IAsDeviceService;
import com.ruoyi.system.service.IEtModelService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 车辆型号Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Slf4j
@Service
public class EtModelServiceImpl implements IEtModelService
{
    @Resource
    private EtModelMapper etModelMapper;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private SysDeptMapper deptMapper;

    @Resource
    private EtModelRuleMapper etModelRuleMapper;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;


    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    @Override
    public EtModel selectEtModelByModelId(Long modelId)
    {
        EtModel etModel = etModelMapper.selectEtModelByModelId(modelId);
        if(ObjectUtil.isNotNull(etModel)){
            Integer allNum = asDeviceService.selectCountByModelId(modelId);
            etModel.setDeviceNum(allNum);
            if(ObjectUtil.isNotNull(etModel.getOperator())){
                List<EtOperatingArea> areaList = etOperatingAreaService.selectAreaListByDeptId2(etModel.getOperator(),null);
                etModel.setAreaList(areaList);
            }
        }
        return etModel;
    }

    /**
     * 查询车辆型号列表
     *
     * @param etModel 车辆型号
     * @return 车辆型号
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtModel> selectEtModelList(EtModel etModel)
    {
        List<EtModel> etModels = etModelMapper.selectEtModelList(etModel);
        for(EtModel model: etModels){
            // 投放数量统计
            AsDevice device = new AsDevice();
            device.setModelId(model.getModelId());
            Integer integer = asDeviceService.selectCountByModelId(model.getModelId());
            model.setDeviceNum(integer);
            // 运营商名称
            SysDept sysDept = deptMapper.selectDeptById(model.getOperator());
            if(ObjectUtil.isNotNull(sysDept)){
               model.setOperatorName(sysDept.getDeptName());
            }
        }
        return etModels;
    }

    /**
     * 新增车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int insertEtModel(EtModel etModel)
    {
        if(etModel.getOperator() == null){
            SysUser currentUser = SecurityUtils.getLoginUser().getUser();//获取当前系统用户
            if(!currentUser.isAdmin()){
                etModel.setOperator(currentUser.getDeptId());
            }
        }
        etModel.setCreateTime(DateUtils.getNowDate());
        int i = etModelMapper.insertEtModel(etModel);
        Long[] ruleIds = etModel.getRuleIds();
        if(ObjectUtil.isNotNull(ruleIds) && ruleIds.length > 0){
            for (Long ruleId:ruleIds){
                etModelRuleMapper.insert(EtModelRule.builder().modelId(etModel.getModelId()).ruleId(ruleId).build());
            }
        }
        // Java报错:No appropriate protocol (protocol is disabled or cipher suites are inappropriate)
        if(!etModel.getLowBatteryReminderSwitch()){// 提醒关闭时，发送低电压命令 为0
            asynchronousSendCommand(etModel.getModelId());
        }

//        // 发送设置低电压命令
//        Integer lowBatteryReminder = etModel.getLowBatteryReminder();
//        if(ObjectUtil.isNotNull(lowBatteryReminder) && lowBatteryReminder > 0){
//            AsDevice device = new AsDevice();
//            device.setModelId(etModel.getModelId());
//            List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(device);
//            for(AsDevice asDevice: asDevices){
//                // 根据百分比计算提醒电压值
//                Integer quantityByPercentage = CommonUtil.getElectricQuantityByPercentage(lowBatteryReminder, etModel.getFullVoltage(), etModel.getLowVoltage());
//                String lowVoltageCommand = IotConstants.COMMAND_BAT + quantityByPercentage * 10 + "@";
//                log.info("发送低电压命令：" + lowVoltageCommand);
//                asDeviceService.sendCommand(asDevice.getMac(), Token.getToken(), lowVoltageCommand,"发送低电压播报",null,null);
//            }
//        }
        return i;
    }

    /**
     * 修改车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int updateEtModel(EtModel etModel)
    {
        if(etModel.getOperator() != null){
            SysUser currentUser = SecurityUtils.getLoginUser().getUser();//获取当前系统用户
            if(!currentUser.isAdmin()){
                etModel.setOperator(currentUser.getDeptId());
            }
        }
        etModel.setUpdateTime(DateUtils.getNowDate());
        int i = etModelMapper.updateEtModel(etModel);
        Long[] ruleIds = etModel.getRuleIds();
        if(ObjectUtil.isNotNull(ruleIds)){
            etModelRuleMapper.deleteModelRuleByModelId(etModel.getModelId());
            if(ObjectUtil.isNotNull(ruleIds)){
                for (Long ruleId:ruleIds){
                    etModelRuleMapper.insert(EtModelRule.builder().modelId(etModel.getModelId()).ruleId(ruleId).build());
                }
            }
        }
        if(!etModel.getLowBatteryReminderSwitch()){// 提醒关闭时，发送低电压命令 为0
            asynchronousSendCommand(etModel.getModelId());
        }
        // 发送设置低电压命令
//        Integer lowBatteryReminder = etModel.getLowBatteryReminder();
//        if(ObjectUtil.isNotNull(lowBatteryReminder) && lowBatteryReminder > 0){
//            AsDevice device = new AsDevice();
//            device.setModelId(etModel.getModelId());
//            List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(device);
//            for(AsDevice asDevice: asDevices){
//                // 根据百分比计算提醒电压值
//                Integer quantityByPercentage = CommonUtil.getElectricQuantityByPercentage(lowBatteryReminder, etModel.getFullVoltage(), etModel.getLowVoltage());
//                String lowVoltageCommand = IotConstants.COMMAND_BAT + quantityByPercentage * 10 + "@";
//                log.info("发送低电压命令：" + lowVoltageCommand);
//                ResponseVo responseVo = asDeviceService.sendCommandWithResp(asDevice.getMac(), Token.getToken(), lowVoltageCommand,"发送低电压播报",null);
//                if(responseVo.getCode()!=0){
//                    log.info("【还车关锁】设备【{}】远程关锁失败", asDevice.getMac());
//                }
//            }
//        }
        return i;
    }

    /**
     * 异步发送低电压命令
     */
    private void asynchronousSendCommand(Long modelId) {
        //开异步线程：
        scheduledExecutorService.schedule(() -> {
            AsDevice device = new AsDevice();
            device.setModelId(modelId);
            List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(device);
            for(AsDevice asDevice: asDevices){
                // 根据百分比计算提醒电压值
                String lowVoltageCommand = IotConstants.COMMAND_BAT + 0 + "@";
                log.info("发送低电压命令：" + lowVoltageCommand);
                ResponseVo responseVo = null;
                try {
                    responseVo = asDeviceService.sendCommandWithResp(asDevice.getMac(), Token.getToken(), lowVoltageCommand,"发送低电压播报",null);
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                } catch (NoSuchAlgorithmException e) {
                    e.printStackTrace();
                } catch (InvalidKeyException e) {
                    e.printStackTrace();
                }
                if(responseVo.getCode()!=0){
                    log.info("【还车关锁】设备【{}】远程关锁失败", asDevice.getMac());
                }
            }
        }, 0, TimeUnit.SECONDS);
    }

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEtModelByModelIds(Long[] modelIds)
    {
        return etModelMapper.deleteEtModelByModelIds(modelIds);
    }

    /**
     * 删除车辆型号信息
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEtModelByModelId(Long modelId)
    {
        return etModelMapper.deleteEtModelByModelId(modelId);
    }

    @Override
    public int selectAllCount() {
        return etModelMapper.selectAllCount();
    }

}
