package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.domain.EtAreaDept;
import com.ruoyi.system.domain.EtAreaRule;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.mapper.EtAreaDeptMapper;
import com.ruoyi.system.mapper.EtAreaRuleMapper;
import com.ruoyi.system.mapper.EtOperatingAreaMapper;
import com.ruoyi.system.service.IAsDeviceService;
import com.ruoyi.system.service.IEtFeeRuleService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtParkingAreaService;
import lombok.extern.slf4j.Slf4j;
import org.locationtech.jts.geom.Coordinate;
import org.locationtech.jts.geom.Geometry;
import org.locationtech.jts.geom.GeometryFactory;
import org.locationtech.jts.geom.Point;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 运营区Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-06
 */
@Slf4j
@Service
public class EtOperatingAreaServiceImpl extends ServiceImpl<EtOperatingAreaMapper, EtOperatingArea> implements IEtOperatingAreaService
{
    @Resource
    private EtOperatingAreaMapper dao;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

//    @Resource
//    private EtAreaRuleMapper etAreaRuleMapper;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private EtAreaDeptMapper etAreaDeptMapper;

    @Resource
    private IEtParkingAreaService etParkingAreaService;


    /**
     * 查询运营区
     *
     * @param areaId 运营区主键
     * @return 运营区
     */
    @Override
    public EtOperatingArea selectEtOperatingAreaByAreaId(Long areaId)
    {
        return dao.selectEtOperatingAreaByAreaId(areaId);
    }

    /**
     * 查询运营区列表
     *
     * @param etOperatingArea 运营区
     * @return 运营区
     */
    @Override
    public List<EtOperatingArea> selectEtOperatingAreaList(EtOperatingArea etOperatingArea)
    {
        List<EtOperatingArea> etOperatingAreas = dao.selectEtOperatingAreaList(etOperatingArea);
        for(EtOperatingArea area : etOperatingAreas){
            Integer integer = asDeviceService.selectCountByAreaId(area.getAreaId());
            area.setDeviceNum(integer);
            List<String> list = etFeeRuleService.selectRuleNameListByAreaId(area.getAreaId());
            StringBuilder ruleStr = new StringBuilder();
            for (String rule : list) {
                if (ruleStr.length() > 0) {
                    ruleStr.append(", ");
                }
                ruleStr.append(rule);
            }
            area.setRuleStr(ruleStr.toString().trim());
        }
        return etOperatingAreas;
    }

    /**
     * 查询运营区列表
     *
     * @param etOperatingArea 运营区 隔离
     * @return 运营区
     */
    @DataScope(deptAlias = "d")
    @Override
    public List<EtOperatingArea> selectEtOperatingAreaListWithisolate(EtOperatingArea etOperatingArea)
    {
        List<EtOperatingArea> etOperatingAreas = dao.selectEtOperatingAreaList(etOperatingArea);
        for(EtOperatingArea area : etOperatingAreas){
            Integer integer = asDeviceService.selectCountByAreaId(area.getAreaId());
            area.setDeviceNum(integer);
            List<String> list = etFeeRuleService.selectRuleNameListByAreaId(area.getAreaId());
            StringBuilder ruleStr = new StringBuilder();
            for (String rule : list) {
                if (ruleStr.length() > 0) {
                    ruleStr.append(", ");
                }
                ruleStr.append(rule);
            }
            area.setRuleStr(ruleStr.toString().trim());
            area.setParkingAreaNum(etParkingAreaService.selectParkingAreaNumByAreaId(area.getAreaId(),ServiceConstants.PARKING_AREA_TYPE_PARKFING));
            area.setNoParkingAreaNum(etParkingAreaService.selectParkingAreaNumByAreaId(area.getAreaId(),ServiceConstants.PARKING_AREA_TYPE_NO_PARKFING));
            area.setBannedRidingAreaNum(etParkingAreaService.selectParkingAreaNumByAreaId(area.getAreaId(),ServiceConstants.PARKING_AREA_TYPE_BANNED_RIDING));
        }
        return etOperatingAreas;
    }

    /**
     * 新增运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    @Transactional
    @Override
    public int insertEtOperatingArea(EtOperatingArea etOperatingArea)
    {
        etOperatingArea.setCreateTime(DateUtils.getNowDate());
        /** 将边界值转Geometry对象*/
        String boundaryStr = etOperatingArea.getBoundaryStr();
        Geometry geometry = GeoUtils.toGeometry(boundaryStr);
        String wkt = GeoUtils.wkt(geometry);
        etOperatingArea.setBoundary(wkt);
        int insert = dao.insert(etOperatingArea);
//        Long[] ruleIds = etOperatingArea.getRuleIds();
//        if(ObjectUtil.isNotNull(ruleIds) && ruleIds.length > 0){
//            for (Long ruleId:ruleIds){
//                etAreaRuleMapper.insert(EtAreaRule.builder().areaId(etOperatingArea.getAreaId()).ruleId(ruleId).build());
//            }
//        }
        Long deptId = SecurityUtils.getDeptId();
        if(deptId == 100){
            deptId = etOperatingArea.getDeptId();
        }
        if(!ObjectUtil.isNotNull(etOperatingArea.getDeptId()) && deptId == 100){
            throw new ServiceException("请选择运营商");
        }
        int insert1 = etAreaDeptMapper.insert(EtAreaDept.builder().areaId(etOperatingArea.getAreaId()).deptId(deptId).build());
        if(insert1 > 0){
            log.info("新增运营区与运营商关联成功");
        }
        return insert;
    }

    /**
     * 修改运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    @Override
    @Transactional
    public int updateEtOperatingArea(EtOperatingArea etOperatingArea)
    {
        /** 将边界值转Geometry对象*/
        String boundaryStr = etOperatingArea.getBoundaryStr();
        Geometry geometry = GeoUtils.toGeometry(boundaryStr);
        String wkt = GeoUtils.wkt(geometry);
        etOperatingArea.setBoundary(wkt);
        int i = dao.updateById(etOperatingArea);
//        Long[] ruleIds = etOperatingArea.getRuleIds();
//        if(ObjectUtil.isNotNull(ruleIds)){
//            etAreaRuleMapper.deleteAreaRuleByAreaId(etOperatingArea.getAreaId());
//            if(ObjectUtil.isNotNull(ruleIds)){
//                for (Long ruleId:ruleIds){
//                    etAreaRuleMapper.insert(EtAreaRule.builder().areaId(etOperatingArea.getAreaId()).ruleId(ruleId).build());
//                }
//            }
//        }
        Long deptId = SecurityUtils.getDeptId();
        if(deptId == 100){
            deptId = etOperatingArea.getDeptId();
        }
        if(!ObjectUtil.isNotNull(etOperatingArea.getDeptId()) && deptId == 100){
            throw new ServiceException("请选择运营商");
        }
        int i1 = etAreaDeptMapper.deleteAreaDeptByAreaId(etOperatingArea.getAreaId());
        if(i1 > 0){
            etAreaDeptMapper.insert(EtAreaDept.builder().areaId(etOperatingArea.getAreaId()).deptId(deptId).build());
        }
        return i;
    }

    /**
     * 修改运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    @Override
    public int updateEtOperatingArea2(EtOperatingArea etOperatingArea)
    {
        String boundaryStr = etOperatingArea.getBoundaryStr();
        Geometry geometry = GeoUtils.toGeometry(boundaryStr);
        String wkt = GeoUtils.wkt(geometry);
        etOperatingArea.setBoundary(wkt);
        int i = dao.updateById(etOperatingArea);
        return i;
    }

    /**
     * 批量删除运营区
     *
     * @param areaIds 需要删除的运营区主键
     * @return 结果
     */
    @Override
    public int deleteEtOperatingAreaByAreaIds(Long[] areaIds)
    {
        for (Long areaId:areaIds) {
            dao.deleteById(areaId);
        }
        return 1;
    }

    /**
     * 删除运营区信息
     *
     * @param areaId 运营区主键
     * @return 结果
     */
    @Override
    public int deleteEtOperatingAreaByAreaId(Long areaId)
    {
        return dao.deleteById(areaId);
    }

    /**
     * 根据所有运营分区
     *
     * @return 字典类型集合信息
     */
    @Override
    public List<EtOperatingArea> selectAreaAll()
    {
        QueryWrapper<EtOperatingArea> query = Wrappers.query();
        query.orderByAsc("area_name");
        return dao.selectList(query);
    }

    /**
     * 校验运营区是否唯一
     *
     * @param etOperatingArea 运营区信息
     * @return 结果
     */
    @Override
    public boolean checkAreaNameUnique(EtOperatingArea etOperatingArea)
    {
        Long areaId = StringUtils.isNull(etOperatingArea.getAreaId()) ? -1L : etOperatingArea.getAreaId();
        QueryWrapper<EtOperatingArea> queryWrapper = Wrappers.query();
        queryWrapper.eq("area_name", etOperatingArea.getAreaName());
        EtOperatingArea info = dao.selectOne(queryWrapper);
        if (StringUtils.isNotNull(info) && info.getAreaId().longValue() != areaId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 根据定位获取运营区信息
     *
     * @param longitude 经度
     * @param latitude 纬度
     * @param deptId 运营商
     * @param sn 设备sn
     * @return 结果
     */
    @Override
    public EtOperatingArea getAreaInfoByLocation(String longitude, String latitude,String deptId,String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        if(StrUtil.isNotBlank(sn) && ObjectUtil.isNotNull(device)){
            return dao.selectById(device.getAreaId());
        }else{
            double lon = Double.parseDouble(longitude);
            double lat = Double.parseDouble(latitude);
            //如果是创享小程序，根据‘是否使用创享电动车小程序’查询出所有为是的运营商下所有的运营区
            EtOperatingArea area1 = new EtOperatingArea();
            if(!"100".equals(deptId)){
                area1.setDeptId(Long.parseLong(deptId));
            }
            List<EtOperatingArea> etOperatingAreas = etOperatingAreaService.selectEtOperatingAreaList(area1);
            EtOperatingArea area = null;
            for(EtOperatingArea etOperatingArea:etOperatingAreas){
                String boundary = etOperatingArea.getBoundary();
                if(StrUtil.isNotBlank(boundary)){
                    Geometry geometry = GeoUtils.fromWkt(boundary);
                    Boolean inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
                    if(inCircle){
                        area = etOperatingArea;
                        break;
                    }
                }
            }
            if(ObjectUtil.isNotNull(area)){
                /** 在运营区内直接返回*/
                log.info("在【{}】运营区内",area.getAreaName());
                return area;
            }else{
                /** 不在任何运营区内，获取到最近一个运营区*/
                log.info("不在任何运营区内,获取到最近一个运营区");
                EtOperatingArea closestArea = null;
                double minDistance = Double.MAX_VALUE;
                GeometryFactory geometryFactory = new GeometryFactory();
                Coordinate coordinate = new Coordinate(lon, lat);
                Point point = geometryFactory.createPoint(coordinate);
                if(ObjectUtil.isNotNull(etOperatingAreas) && etOperatingAreas.size()>0){
                    for(EtOperatingArea etOperatingArea:etOperatingAreas){
                        String boundary = etOperatingArea.getBoundary();
                        if(StrUtil.isNotBlank(boundary)){
                            Geometry geometry = GeoUtils.fromWkt(etOperatingArea.getBoundary());
                            if (geometry.contains(point)) {
                                return etOperatingArea;
                            }else{
                                // 获取多边形的外边界
                                Coordinate[] coordinates = geometry.getCoordinates();
                                for (Coordinate coord : coordinates) {
                                    double distance = GeoUtils.calculateDistance(lat, lon, coord.y, coord.x);
                                    if (distance < minDistance) {
                                        minDistance = distance;
                                        closestArea = etOperatingArea;
                                    }
                                }
                            }
                        }else{
                            log.info("运营区【{}】没有边界",etOperatingArea.getAreaName());
                        }
                    }
                }else{
                    throw new ServiceException("没有找到任何运营区");
                }
                return closestArea;
            }
        }

    }

    /**
     * 根据部门ID获取运营区选择框列表
     *
     * @param deptId 用户ID
     * @return 选中运营区ID列表
     */
    @Override
    public List<Long> selectAreaListByDeptId(Long deptId) {
        return dao.selectAreaListByDeptId(deptId);
    }

    /**
     * 根据部门ID获取运营区选择框列表
     *
     * @param deptId 用户ID
     * @return 选中运营区ID列表
     */
    @Override
    public List<EtOperatingArea> selectAreaListByDeptId2(Long deptId,Long areaId) {
        return dao.selectAreaListByDeptId2(deptId,areaId);
    }

    /**
     * 查询运营区是否存在车辆
     *
     * @param areaId 区域id
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistDevice(Long areaId)
    {
        Integer result = asDeviceService.selectCountByAreaId(areaId);
        return result > 0;
    }


}
