package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.enums.BusinessStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.PaymentResult;
import com.ruoyi.common.pay.tm.TmPayService;
import com.ruoyi.common.pay.tm.enums.PayStatus;
import com.ruoyi.common.pay.tm.vo.TmTradeInfo;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.onenet.ResponseVo;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.*;
import com.ruoyi.system.mapper.*;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 订单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@Slf4j
@Service
public class EtOrderServiceImpl implements IEtOrderService
{
    @Resource
    private EtOrderMapper etOrderMapper;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private IAsUserService asUserService;

    @Resource
    private AsUserMapper asUserMapper;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IAsDeviceService deviceService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtRefundService etRefundService;

    @Autowired
    private IEtTripLogService etTripLogService;

    @Autowired
    private CallbackService callbackService;

    @Autowired
    private ISysDeptService deptService;

    @Resource
    private EtRefundMapper etRefundMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Resource
    private IEtModelService etModelService;

    @Autowired
    private IEtCapitalFlowService etCapitalFlowService;

    @Resource
    private EtFaultMapper etFaultMapper;

    @Resource
    private EtAdminOrderMapper etAdminOrderMapper;

    @Resource
    private EtCapitalFlowMapper etCapitalFlowMapper;

    @Autowired
    private ISysOperLogService operLogService;

    @Resource
    private EtCouponMapper etCouponMapper;

    @Resource
    private EtCouponClaimLogMapper etCouponClaimLogMapper;

    @Autowired
    private EtChannelService etChannelService;

    @Autowired
    private TmPayService tmPayService;

    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public EtOrder selectEtOrderByOrderId(Long orderId)
    {
        // todo 订单详情获取行程记录、费用明细、退款明细、收费方式、终点起点、
        EtOrder order = etOrderMapper.selectEtOrderByOrderId(orderId);
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
        order.setDevice(device);
        EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleIdIncludeDelete(order.getRuleId());
        setDescription(order);
        String endTime;
        if(ObjectUtil.isNull(order.getReturnTime())){
            endTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, DateUtils.getNowDate());
        }else{
            endTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, order.getReturnTime());
        }
        String startTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, order.getUnlockTime());
        String trajectory = deviceService.trajectory(order.getSn(), startTime, endTime);
        order.setTripRouteStr(trajectory);
        order.setRule(etFeeRule);
        //行程记录
        EtTripLog tripLog = new EtTripLog();
        tripLog.setOrderNo(order.getOrderNo());
        List<EtTripLog> tripLogs = etTripLogService.selectEtTripLogList(tripLog);
        if(ObjectUtils.isNotEmpty(tripLogs)){
            order.setTripLogs(tripLogs);
        }
        // 退款记录
        List<EtRefund> refunds = etRefundService.selectEtRefundByOrderNo(order.getOrderNo());
        if(ObjectUtils.isNotEmpty(refunds) && refunds.size() > 0){
            order.setEtRefunds(refunds);
        }
        return order;
    }

    private void setDescription(EtOrder order) {
        StringBuilder sb = new StringBuilder();
        String ridingRuleJson = order.getRidingRuleJson();
        String unit = "";
        if(order.getRidingRule() != null && order.getRidingRule().equals("1")){
            StartingRuleVo startingRule = JSONObject.parseObject(ridingRuleJson, StartingRuleVo.class);
            String rentalUnit = order.getRentalUnit();
            if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_MINUTES)){
                unit = "分钟";
            }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){
                unit = "小时";
            }
            sb.append("起步价："+startingRule.getStartingPrice()+"元(含"+startingRule.getStartingTime()+unit+"),");
            sb.append("超出价："+startingRule.getTimeoutPrice()+"元/"+startingRule.getTimeoutTime()+unit);
        }
        order.setDescription(sb.toString());
    }

    /**
     * 根据订单号查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public EtOrder selectEtOrderByOrderNo(String orderNo)
    {
        return etOrderMapper.selectEtOrderByOrderNo(orderNo);
    }

    /**
     * 根据微信内部订单号查询订单
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    @Override
    public EtOrder selectEtOrderByOutTradeNo(String outTradeNo)
    {
        return etOrderMapper.selectEtOrderByOutTradeNo(outTradeNo);
    }

    /**
     * 根据订单号查询订单(详细)
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public EtOrder getOrderInfo(String orderNo)
    {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(orderNo);
        if (ObjectUtils.isEmpty(order)) {
            throw new ServiceException("订单不存在");
        }
        if (ObjectUtils.isNotEmpty(order)) {
            AsUser user = asUserService.selectUserById(order.getUserId());
            if(ObjectUtils.isNotEmpty(user)){
                order.setPhonenumber(user.getPhonenumber());
                order.setUserName(user.getUserName());
                order.setRealName(user.getRealName());
            }
            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
            if(ObjectUtils.isNotEmpty(area)){
                order.setArea(area.getAreaName());
                order.setLongitude(area.getLongitude());
                order.setLatitude(area.getLatitude());
            }
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            order.setDevice(device);
            //根据规则id查询规则
            EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleIdIncludeDelete(order.getRuleId());
            if(ObjectUtils.isNotEmpty(etFeeRule)){
                order.setRule(etFeeRule);
            }
            //退款记录
            // 初始化可退金额为订单的原始费用
            BigDecimal remainingDispatchFee = order.getDispatchFee();
            BigDecimal remainingManageFee = order.getManageFee();
            BigDecimal remainingRidingFee = order.getRidingFee();
            BigDecimal remainingAppointmentFee = order.getAppointmentFee();
            BigDecimal totalRefundedAmount = BigDecimal.ZERO; // 总退款金额

            List<EtRefund> refundList = etRefundService.selectEtRefundByOrderNo(order.getOrderNo());
            if(ObjectUtils.isNotEmpty(refundList) && refundList.size() > 0){
                order.setEtRefunds(refundList);
            }
            if(ObjectUtils.isNotEmpty(refundList)){
                for (EtRefund refund : refundList) {
                    totalRefundedAmount = totalRefundedAmount.add(refund.getAmount() != null ? refund.getAmount() : BigDecimal.ZERO);
                    // 减去已退款的金额
                    remainingDispatchFee = remainingDispatchFee.subtract(refund.getDispatchFee() != null ? refund.getDispatchFee() : BigDecimal.ZERO);
                    remainingManageFee = remainingManageFee.subtract(refund.getManageFee() != null ? refund.getManageFee() : BigDecimal.ZERO);
                    remainingRidingFee = remainingRidingFee.subtract(refund.getRidingFee() != null ? refund.getRidingFee() : BigDecimal.ZERO);
                    remainingAppointmentFee = remainingAppointmentFee.subtract(refund.getAppointmentFee() != null ? refund.getAppointmentFee() : BigDecimal.ZERO);
                }
                order.setSettlementFee(order.getTotalFee().subtract(totalRefundedAmount));
            }else{
                order.setSettlementFee(order.getTotalFee());
            }
            order.setRefundableDispatchFee(remainingDispatchFee.max(BigDecimal.ZERO));
            order.setRefundableManageFee(remainingManageFee.max(BigDecimal.ZERO));
            order.setRefundableRideFee(remainingRidingFee.max(BigDecimal.ZERO));
            order.setRefundableAppointmentFee(remainingAppointmentFee.max(BigDecimal.ZERO));

            //行程记录
            EtTripLog tripLog = new EtTripLog();
            tripLog.setOrderNo(order.getOrderNo());
            List<EtTripLog> tripLogs = etTripLogService.selectEtTripLogList(tripLog);
            if(ObjectUtils.isNotEmpty(tripLogs)){
                order.setTripLogs(tripLogs);
            }
        }
        return order;
    }

    /**
     * 查询订单列表
     * @param etOrder 订单
     * @return 订单  已结束
     */
    @Override
    public List<EtOrder> selectEtOrderList(EtOrder etOrder)
    {
        String[] statusList = etOrder.getStatusList();
        if(ObjectUtil.isNotNull(statusList) && toBePaid(statusList)){
            etOrder.setPaid("0");
        }
        if((ObjectUtil.isNotNull(statusList) && ended(statusList)) || ServiceConstants.ORDER_TYPE_DEPOSIT.equals(etOrder.getType()) ){
            etOrder.setPaid("1");
        }
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        //如果查询押金则增加退款记录
//        if(ServiceConstants.ORDER_TYPE_DEPOSIT.equals(etOrder.getType())){
        etOrders.forEach(etOrder1 -> {
            List<EtRefund> refund = etRefundService.selectEtRefundByOrderNo(etOrder1.getOrderNo());
            if(ObjectUtil.isNotNull(refund) && refund.size() > 0){
                etOrder1.setEtRefund(refund.get(0));
            }
        });
//        }
        return etOrders;
    }

    /**
     * 查询订单列表
     * @param etOrder 订单
     * @return 订单  已结束
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtOrder> selectEtOrderListWithIsolate(EtOrder etOrder)
    {
        String[] statusList = etOrder.getStatusList();
        if(ObjectUtil.isNotNull(statusList) && toBePaid(statusList)){
            etOrder.setPaid("0");
        }
        if((ObjectUtil.isNotNull(statusList) && ended(statusList)) || ServiceConstants.ORDER_TYPE_DEPOSIT.equals(etOrder.getType()) ){
            etOrder.setPaid("1");
        }
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        //如果查询押金则增加退款记录
        etOrders.forEach(etOrder1 -> {
            String orderNo;
            if(etOrder1.getDepositDeduction().equals(ServiceConstants.IS_DEPOSIT_DEDUCTION) && etOrder1.getStatus().equals(ServiceConstants.ORDER_STATUS_ORDER_END)){
                orderNo = etOrder1.getDepositOrderNo();
            }else{
                orderNo = etOrder1.getOrderNo();
            }
            List<EtRefund> refunds = etRefundService.selectEtRefundByOrderNo(orderNo);
            if(ObjectUtil.isNotNull(refunds) && refunds.size() > 0){
                etOrder1.setEtRefund(refunds.get(0));
                etOrder1.setEtRefunds(refunds);
                // 计算总的退款金额
                BigDecimal totalRefundFee = refunds.stream()
                        .map(EtRefund::getAmount)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                etOrder1.setTotalRefundFee(totalRefundFee);
            }
        });
        return etOrders;
    }

    /**
     * 查询充值/退款订单列表
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<RechargeVo> rechargeList(EtOrderQuery etOrder) {
        List<RechargeVo> rechargeVoList = new ArrayList<>();
        String status = etOrder.getStatus();
        if(StrUtil.isBlank(status) || "1".equals(status)){
            etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
            List<EtOrder> etOrders = etOrderMapper.selectEtOrderListNoRoute2(etOrder);
            etOrders.forEach(etOrder1 -> {
                RechargeVo rechargeVo = new RechargeVo();
                BeanUtils.copyProperties(etOrder1,rechargeVo);
                rechargeVo.setRechargeStatus(ServiceConstants.RECHARGE_STATUS_SUCCESS);
                rechargeVo.setRechargeType(ServiceConstants.ORDER_TYPE_DEPOSIT);
                rechargeVo.setTotalFee(etOrder1.getPayFee());
                rechargeVo.setDepositDeduction(etOrder1.getDepositDeduction());
                List<EtRefund> etRefunds = etRefundMapper.selectEtRefundByOrderNo(etOrder1.getOrderNo());
                if(ObjectUtil.isNull(etRefunds) || etRefunds.size() == 0){
                    rechargeVo.setIsRefunded(false);
                }else{
                    rechargeVo.setIsRefunded(true);
                }
                rechargeVoList.add(rechargeVo);
            });
        }
        if(StrUtil.isBlank(status) || "2".equals(status)){
            //如果查询押金则增加退款记录
            etOrder.setRefundType(ServiceConstants.REFUND_TYPE_DEPOSIT);
            etOrder.setRefundResult(Constants.SUCCESS2);
            PageUtils.startPage();
            List<EtRefund> etRefunds = etRefundMapper.selectEtRefundList2(etOrder);
            etRefunds.forEach(etRefund -> {
                AsUser asUser = asUserMapper.selectUserById(etRefund.getUserId());
                RechargeVo rechargeVo = new RechargeVo();
                BeanUtils.copyProperties(etRefund,rechargeVo);
                rechargeVo.setOutTradeNo(etRefund.getRefundNo());
                rechargeVo.setPayTime(etRefund.getCreateTime());
                rechargeVo.setTotalFee(etRefund.getAmount());
                rechargeVo.setUserId(etRefund.getUserId());
                rechargeVo.setUserName(asUser.getUserName());
                rechargeVo.setPhonenumber(asUser.getPhonenumber());
                rechargeVo.setRechargeStatus(ServiceConstants.RECHARGE_STATUS_REFUND_SUCCESS);
                rechargeVo.setRechargeType(ServiceConstants.ORDER_TYPE_DEPOSIT_REFUND);
                rechargeVoList.add(rechargeVo);
            });
        }
        //将rechargeVoList根据payTime倒序
        rechargeVoList.sort(Comparator.comparing(RechargeVo::getPayTime).reversed());
        return rechargeVoList;
    }

    /**
     * 获取当前正在骑行中的订单
     * @param sn
     * @return
     */
    @Override
    public EtOrder getCurrentOrder(String sn) {
        if(ObjectUtil.isNotNull(asDeviceMapper.selectAsDeviceBySn(sn))){
            EtOrder etOrder = new EtOrder();
            etOrder.setSn(sn);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
            List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
            if(ObjectUtil.isNotEmpty(etOrders)){
                return etOrders.get(0);
            }
        }
        return null;
    }

    /**
     * 获取当前正在骑行中的订单
     * @param sn
     * @return
     */
    @Override
    public EtOrder getCurrentOrder2(String sn) {
        EtOrder etOrder = new EtOrder();
        etOrder.setSn(sn);
        etOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderListNoRoute(etOrder);
        if(ObjectUtil.isNotEmpty(etOrders)){
            return etOrders.get(0);
        }
        return null;
    }

    /**
     * 查询所有正在骑行中的订单
     * @return
     */
    @Override
    public List<EtOrder> getCurrentOrderList() {
        List<EtOrder> etOrders = etOrderMapper.getCurrentOrderList();
        return etOrders;
    }


    /**
     * 根据userId获取当前正在骑行中的订单
     * @param userId
     * @return
     */
    @Override
    public EtOrder getCurrentOrderByUserId(Long userId) {
        EtOrder etOrder = new EtOrder();
        etOrder.setUserId(userId);
        etOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        if(ObjectUtil.isNotEmpty(etOrders)){
            return etOrders.get(0);
        }
        return null;
    }

    /**
     * 删除订单信息
     *
     * @param orderNo 订单号
     * @return 结果
     */
    @Override
    public int deleteEtOrderByOrderNo(String orderNo) {
        if(ObjectUtil.isNotNull(etOrderMapper.selectEtOrderByOrderNo(orderNo))){
            return etOrderMapper.deleteEtOrderByOrderNo(orderNo);
        }
        return 1;
    }

    /**
     * 押金抵扣
     * 1. 校验订单状态为待支付，不是待支付则返回false
     * 2. 根据当前订单号查询订单信息
     * 3. 获取用户最后一次押金充值订单
     * 4. 将抵扣后的金额 = 押金-骑行订单金额
     *      如果金额大于0，
     *      如果金额小于0，押金订单金额 = 0，更新骑行订单状态为未支付，
     * 5. 更新押金订单金额（做一个押金抵扣的标记）
     * 6. 更新骑行订单状态为已支付，支付方式为 押金抵扣
     * 7. 更新车辆信息
     * 8. 更新用户余额
     */
    @Override
    @Transactional
    public int deduction(EtOrder etOrder) {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(etOrder.getOrderNo());
        EtOrder order1 = new EtOrder();
        order1.setOrderId(order.getOrderId());
        order1.setOrderNo(order.getOrderNo());
        if(!ServiceConstants.ORDER_STATUS_RIDING_END.equals(order.getStatus())){
            throw new ServiceException("押金抵扣失败，订单非待支付状态，订单状态："+order.getStatus());
        }
        // 押金抵扣时，判断是否有outtradeno
        if(StrUtil.isNotBlank(order.getOutTradeNo())){
            String outTradeNo = order.getOutTradeNo();
            log.info("【押金抵扣】订单【{}】,有outTradeNo = 【{}】", order.getOrderNo(),outTradeNo);
            ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(order.getPayChannel());
            // 如果原来有outtradeno，去查询一次，查询是否支付过
            PaymentResult paymentResult = wxPayService.queryOrderByOutTradeNo(outTradeNo);
            try {
                if (ObjectUtil.isNotNull(paymentResult.getTransaction())) {
                    if (Transaction.TradeStateEnum.SUCCESS.equals(paymentResult.getTransaction().getTradeState())) {
                        handleSuccess(order);
                        return 1;
                    }else{
                        // 没有支付，则关闭订单
                        boolean b = wxPayService.closeOrder(outTradeNo,channelVO);
                        log.info("【押金抵扣】订单【{}】,有outTradeNo = 【{}】,查询订单未支付，关闭订单：{}", order.getOrderNo(),outTradeNo,b);
                    }
                }else{
                    TmTradeInfo tradeInfo = paymentResult.getTradeInfo();
                    if(PayStatus.isSuccess(tradeInfo.getPayStatus().getCode())) {
                        handleSuccess(order);
                        return 1;
                    }else{
                        // 没有支付，则关闭订单
                        tmPayService.closeOrder(channelVO,outTradeNo);
                        log.info("【押金抵扣】订单【{}】,有outTradeNo = 【{}】,查询订单未支付，关闭订单：{}", order.getOrderNo(),outTradeNo);
                    }
                }
            } catch (ServiceException e) {
                log.error("异常发生: {}", e.getMessage());
                return -1; // -1 表示失败
            } finally {
                log.warn("订单已支付,请勿操作");
            }
        }
        EtOrder depositOrder = getDepositOrder(order.getUserId());
        BigDecimal deposit = depositOrder.getTotalFee();
        BigDecimal ridingFee = order.getTotalFee();
        BigDecimal afterDeductionFee;
        String mark;
        if(deposit.compareTo(ridingFee) <= 0){
            afterDeductionFee = BigDecimal.ZERO;
            mark = "押金抵扣成功，骑行费【"+ridingFee+"】大于押金【"+deposit+"】";
            order1.setPayFee(deposit);//实际支付金额等于押金
        }else{
            // 押金大于订单金额  扣除后
            afterDeductionFee = deposit.subtract(ridingFee);
            mark = "押金抵扣成功，骑行费【"+ridingFee+"】小于押金【"+deposit+"】,扣除后金额【"+afterDeductionFee+"】";
        }
        /** 更新骑行订单*/
        order1.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        order1.setPayTime(DateUtils.getNowDate());
        order1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        order1.setPayType(ServiceConstants.PAY_TYPE_YJ);
        order1.setMark(mark);
        order1.setDepositDeduction(ServiceConstants.IS_DEPOSIT_DEDUCTION);
        int updateEtOrder = etOrderMapper.updateEtOrder(order1);
        if(updateEtOrder == 0){
            throw new ServiceException("押金抵扣失败，更新骑行订单失败");
        }
        /** 更新押金订单*/
        depositOrder.setDepositDeduction(ServiceConstants.IS_DEPOSIT_DEDUCTION);
        int updateEtOrder1 = etOrderMapper.updateEtOrder(depositOrder);
        if(updateEtOrder1 == 0){
            throw new ServiceException("押金抵扣失败，更新押金订单失败");
        }
        if(afterDeductionFee.compareTo(BigDecimal.ZERO) > 0){
            String refNo = IdUtils.getOrderNo("ref");
            /** 退款剩余押金*/
            wxPayService.refund(depositOrder, "押金抵扣退款",afterDeductionFee,refNo);
            /** 2.记录退款表  创建退款对象*/
            depositOrder.setReason("押金抵扣退款");
            EtRefund refund1= createRefund(depositOrder, afterDeductionFee, null, null, null, null, refNo,ServiceConstants.REFUND_TYPE_DEPOSIT);
            int i = etRefundService.insertEtRefund(refund1);
            if(i == 0){
                log.info("【押金抵扣】保存退款对象失败");
                throw new ServiceException("【押金抵扣】，保存退款对象失败");
            }
        }
        /** 更新用户余额*/
        AsUser asUser = asUserService.selectUserById(depositOrder.getUserId());
        if(asUser!=null){
            // 更新用户并更新缓存
            asUser.setBalance(BigDecimal.ZERO);
            if (asUserService.updateUserProfile(asUser) > 0)
            {
                log.info("【押金抵扣成功】更新用户信息成功:"+ JSON.toJSON(asUser));
            }else{
                throw new ServiceException("【押金抵扣】，更新用户信息失败");
            }
        }
        /** 押金抵扣后生成资金流水记录 */
        callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_RIDING,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YJ);
        return 1;
    }

    private void handleSuccess(EtOrder order) {
        // 订单已支付
        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        order.setPayTime(DateUtils.getNowDate());
        order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        order.setPayType(ServiceConstants.PAY_TYPE_WX);
        order.setMark("主动查询-骑行支付");
        int updateEtOrder = etOrderMapper.updateEtOrderByOrderNo(order);
        if (updateEtOrder == 0) {
            // 抛出自定义异常来标识订单更新失败
            throw new ServiceException("押金抵扣失败，更新骑行订单失败");
        }
    }

    /**
     * 修复押金抵扣
     * 1. 查询所有押金抵扣的订单
     * 2. 如果是骑行费大于押金，则按押金计算，
     *      如果骑行费小于押金，则按骑行费计算
     * 3. 增加一条资金流水记录
     */
    @Override
    @Transactional
    public int repairDeduction(String startDateStr,String endDateStr,Long areaId) {
        List<EtOrder> orders = etOrderMapper.selectDeductionList(startDateStr, endDateStr,areaId);
        for (EtOrder order : orders) {
            // 找出所有骑行费大于押金的订单，把骑行费改成押金
            if(order.getMark().contains("大于")){
                String input = order.getMark();
                // 正则表达式，匹配“押金”后的金额
                String regex = "押金【(\\d+\\.\\d+)】";
                // 编译正则表达式
                Pattern pattern = Pattern.compile(regex);
                Matcher matcher = pattern.matcher(input);
                // 查找匹配项
                if (matcher.find()) {
                    // 提取金额
                   String depositAmount = matcher.group(1);
                   order.setPayFee(new BigDecimal(depositAmount));
                }
            }
            /** 押金抵扣后生成资金流水记录 */
            callbackService.capitalFlowRecordsByRepair(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_RIDING,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YJ);
        }
        return 1;
    }

    /**
     * 还车审核通过
     *  1.更新订单状态为订单结束
     *  2.退押金
     *  3.创建退款记录
     *  4.更新用户余额
     */
    @Override
    public boolean passAudit(EtOrder etOrder) {
        /** 1.更新订单状态为订单结束*/
        EtOrder etOrder1 = new EtOrder();
        etOrder1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        etOrder1.setOrderId(etOrder.getOrderId());
        int updateEtOrder = etOrderMapper.updateEtOrder(etOrder1);
        if(updateEtOrder == 0){
          throw new ServiceException("还车审核失败，更新订单失败");
        }
        // 判断最后一个押金订单是否已退款，如果没有退款，则发起退款，如果有退款，则不退押金
        // 判断是否已退款
        Boolean refunded = isRefunded(etOrder.getUserId());

        // 当前还有正在骑行中的订单，不退押金，只有用户没有正在骑行中的订单时才发起退款
        EtOrder currentOrder = etOrderService.getCurrentOrderByUserId(etOrder.getUserId());
        if(ObjectUtil.isNull(currentOrder) && !refunded){
            log.info("【还车审核通过】当前用户没有正在骑行的订单，开始退押金");
            /** 2.退押金*/
            EtOrder depositOrder = getDepositOrder(etOrder.getUserId());
            BigDecimal deposit = depositOrder.getTotalFee();
            String refNo = IdUtils.getOrderNo("ref");
            wxPayService.refund(depositOrder, "还车审核通过后退押金",deposit,refNo);
            /** 3.记录退款表  创建退款对象*/
            depositOrder.setReason("还车审核通过后退押金");
            EtRefund refund1= createRefund(depositOrder, deposit, null, null, null, null, refNo,ServiceConstants.REFUND_TYPE_DEPOSIT);
            int i = etRefundService.insertEtRefund(refund1);
            if(i == 0){
                log.info("【还车审核通过】保存退款对象失败");
                throw new ServiceException("【还车审核通过】，保存退款对象失败");
            }
            /** 4.更新用户余额*/
            AsUser asUser = asUserService.selectUserById(depositOrder.getUserId());
            if(asUser!=null){
                // 更新用户并更新缓存
                asUser.setBalance(BigDecimal.ZERO);
                if (asUserService.updateUserProfile(asUser) > 0)
                {
                    log.info("【还车审核通过】更新用户信息成功:"+ JSON.toJSON(asUser));
                }else{
                    throw new ServiceException("【还车审核通过】，更新用户信息失败");
                }
            }
        }else{
            if(refunded){
                log.info("【还车审核通过】当前用户押金已退款！！------不退押金");
            }
            if(ObjectUtil.isNull(currentOrder)){
                log.info("【还车审核通过】当前用户还有正在骑行的订单------不退押金");
            }
        }
        return true;
    }

    /** 判断是否已退款*/
    private Boolean isRefunded(Long userId) {
        EtOrder etOrder = new EtOrder();
        etOrder.setUserId(userId);
        etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
        etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        if (etOrders.size() > 0 || ObjectUtil.isNotNull(etOrders) ) {
            Optional<EtOrder> latestOrderOptional = etOrders.stream()
                    .max(Comparator.comparing(EtOrder::getPayTime));
            if (latestOrderOptional.isPresent()) {
                EtOrder lastOrder = latestOrderOptional.get();
                log.info("【还车审核通过】用户【{}】最后一次押金充值订单 : " + JSON.toJSONString(lastOrder),userId);
                // 根据最新的订单号，查询是否有退款记录
                List<EtRefund> etRefunds = etRefundService.selectEtRefundByOrderNo(lastOrder.getOrderNo());
                if(etRefunds.size() == 1) {
                    EtRefund etRefund = etRefunds.get(0);
                    // 有退款记录，判断是否成功
                    if(Constants.SUCCESS2.equals(etRefund.getRefundResult())){
                        log.info("【还车审核通过】押金退款成功回调，退款单号：【{}】",etRefund.getRefundNo());
                        return true;
                    }
                }
            }
        }
        return false;
    }

    /**
     * 官方审核通过
     *  1.更新订单状态为订单结束
     *  2.退押金
     *  3.创建退款记录
     *  4.更新用户余额
     */
    @Override
    public boolean authoritypass(EtOrder etOrder) {
        /** 1.更新订单状态为订单结束*/
        EtOrder etOrder1 = new EtOrder();
        etOrder1.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        etOrder1.setOrderId(etOrder.getOrderId());
        int updateEtOrder = etOrderMapper.updateEtOrder(etOrder1);
        if(updateEtOrder == 0){
            throw new ServiceException("还车审核失败，更新订单失败");
        }
        Boolean refunded = isRefunded(etOrder.getUserId());
        if(!refunded){
            /** 2.退剩余押金 = 押金 - 扣除金额*/
            EtOrder depositOrder = getDepositOrder(etOrder.getUserId());
            BigDecimal deductionAmount = new BigDecimal(etOrder.getDeductionAmount());
            BigDecimal residualDeposit = depositOrder.getTotalFee().subtract(deductionAmount);//抵扣后的金额
            String refNo = IdUtils.getOrderNo("ref");
            wxPayService.refund(depositOrder, "还车审核通过后退押金",residualDeposit,refNo);
            /** 3.记录退款表  创建退款对象*/
            depositOrder.setReason("还车审核通过后退押金");
            if(deductionAmount.compareTo(BigDecimal.ZERO) > 0){
                EtRefund refund1= createRefund(depositOrder, residualDeposit, null, null, null, null, refNo,ServiceConstants.REFUND_TYPE_DEPOSIT);
                int i = etRefundService.insertEtRefund(refund1);
                if(i == 0){
                    log.info("【还车审核通过】保存退款对象失败");
                    throw new ServiceException("【还车审核通过】，保存退款对象失败");
                }
            }
            /** 4.更新用户余额*/
            AsUser asUser = asUserService.selectUserById(depositOrder.getUserId());
            if(asUser!=null){
                // 更新用户并更新缓存
                asUser.setBalance(BigDecimal.ZERO);
                if (asUserService.updateUserProfile(asUser) > 0)
                {
                    log.info("【还车审核通过】更新用户信息成功:"+ JSON.toJSON(asUser));
                }else{
                    throw new ServiceException("【还车审核通过】，更新用户信息失败");
                }
            }
            if(deductionAmount.compareTo(BigDecimal.ZERO) >= 0){//抵扣金额大于0时，增加车损收入
                /** 押金抵扣后生成资金流水记录 */
                etOrder.setPayFee(deductionAmount);
                callbackService.capitalFlowRecords(etOrder,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_VEHICLE_DAMAGE,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YJ);
            }
        }else{
            log.info("【还车审核通过】当前用户押金已退款！！------不退押金");
        }
        return true;
    }


    public EtOrder getDepositOrder(Long userId) {
        EtOrder depositOrder = new EtOrder();
        depositOrder.setUserId(userId);
        depositOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        depositOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
        depositOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(depositOrder);
        if (etOrders.size() == 0 || ObjectUtil.isNull(etOrders) ) {
            throw new ServiceException("押金充值记录不存在");
        }
        Optional<EtOrder> latestOrder = etOrders.stream()
                .max(Comparator.comparing(EtOrder::getPayTime));
        if (latestOrder.isPresent()) {
            EtOrder newestOrder = latestOrder.get();
            log.info("【押金抵扣】最后一次押金充值记录 : " + JSON.toJSONString(newestOrder));
            return newestOrder;
        }
        throw new ServiceException("押金充值记录不存在");
    }

    /**
     * 首页统计-运营商
     */
    @Override
    public IndexVo statistics(Long areaId) {
        /** 首页数据*/
        SysDept sysDept = wxPayService.getDeptObjByAreaId(areaId);
        IndexVo indexVo = createIndexVo(areaId,sysDept);

        /** 营收统计*/
        ArrayList<IndexVo.IncomeVo> incomeVos = new ArrayList<>();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.getNowDate());

        for (int i = 0; i < 13; i++) {
            String formattedDate = dateFormat.format(calendar.getTime());
            String startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
            String endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;
            IndexVo.OrderFeeStatisticsVo todayOrderInfo = getOrderFeeStatistics(startDateStr, endDateStr, areaId);
            log.info("【首页统计】营收统计----------todayOrderInfo: {}" , JSON.toJSON(todayOrderInfo));
            IndexVo.IncomeVo incomeVo = new IndexVo.IncomeVo();
            incomeVo.setDay(formattedDate);
            incomeVo.setOrderNum(todayOrderInfo.getOrderCount());
            incomeVo.setIncome(todayOrderInfo.getIncomeFee());
            incomeVos.add(incomeVo);
            calendar.add(Calendar.DATE, -1);
        }

        indexVo.setIncomeVoList(incomeVos);

        /** 车辆统计*/
        IndexVo.VehicleVo vehicleVo = new IndexVo.VehicleVo();
        DeviceNumVo deviceNumVo = asDeviceService.allVehicleNum("0","100",areaId+"");
        com.ruoyi.common.utils.bean.BeanUtils.copyBeanProp(vehicleVo,deviceNumVo);
        indexVo.setVehicleVo(vehicleVo);

        /** 运维统计*/
        IndexVo.OperationVo operationVo = new IndexVo.OperationVo();

        operationVo.setReplacementOrderCount(etAdminOrderMapper.getOrderCount(ServiceConstants.ADMIN_ORDER_TYPE_REPLACEMENT,ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_UNDER_WAY,areaId));
        operationVo.setCompletedReplacementOrderCount(etAdminOrderMapper.getOrderCount(ServiceConstants.ADMIN_ORDER_TYPE_REPLACEMENT,ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_COMPLETED,areaId));

        operationVo.setDispatchCount(0);// 暂时没有调度工单
        operationVo.setCompletedDispatchCount(0);// 暂时没有调度工单

        operationVo.setRepairCount(etAdminOrderMapper.getOrderCount(ServiceConstants.ADMIN_ORDER_TYPE_REPAIR,ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_REPAIR,areaId));
        operationVo.setCompletedRepairCount(etAdminOrderMapper.getOrderCount(ServiceConstants.ADMIN_ORDER_TYPE_REPAIR,ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_COMPLETED,areaId));
        indexVo.setOperationVo(operationVo);
        log.info("【首页统计】indexVo==={}",JSON.toJSON(indexVo));
        return indexVo;
    }

    private IndexVo createIndexVo(Long areaId,SysDept sysDept) {
        IndexVo indexVo = new IndexVo();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.getNowDate());
        String formattedDate = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
        String startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
        String endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;

        indexVo.setBalance(sysDept.getBalance());//余额
        BigDecimal withdrawFee = etCapitalFlowService.getWithdrawnFee(areaId,ServiceConstants.FLOW_STATUS_COMPLETE);
        BigDecimal applyFee = etCapitalFlowService.getWithdrawnFee(areaId,ServiceConstants.FLOW_STATUS_APPLY);
        indexVo.setWithdraw(defaultIfNull(withdrawFee, BigDecimal.ZERO));//已提现
        indexVo.setSettlementAmount(defaultIfNull(applyFee, BigDecimal.ZERO));//待结算金额

        IndexVo.OrderFeeStatisticsVo todayOrderInfo = getOrderFeeStatistics(startDateStr, endDateStr, areaId);
        indexVo.setTodayIncome(todayOrderInfo.getIncomeFee());//今日营收
        indexVo.setTodayOrderFee(todayOrderInfo.getOrderFee());//今日订单金额
        indexVo.setTodayOrderCount(todayOrderInfo.getOrderCount());//今日订单数

        IndexVo.OrderFeeStatisticsVo totalOrderInfo = getOrderFeeStatistics(null, null, areaId);
        indexVo.setTotalIncome(totalOrderInfo.getIncomeFee());//总营收
        indexVo.setTotalOrderFee(totalOrderInfo.getOrderFee());//总订单金额
        indexVo.setTotalOrderCount(totalOrderInfo.getOrderCount());//总订单数

        indexVo.setUnpaidOrderCount(Integer.parseInt(etOrderMapper.getUnpaidOrder(null,null,areaId+"")));//待付款订单数
        indexVo.setUnpaidOrderFee(new BigDecimal(etOrderMapper.getTotalUnpaid(null,null,areaId+"")));//待付款订单金额
        indexVo.setReturnOrderCount(etOrderMapper.getAuditOrderNum(null,null,areaId+""));//还车待审核订单
        indexVo.setRidingOrder(Integer.parseInt(etOrderMapper.getRidingOrder(null,null,areaId+"")));//骑行中订单
        indexVo.setTodayRefundFee(new BigDecimal(etOrderMapper.getTotalRefund(startDateStr,endDateStr,areaId+"")));// 今日退款金额
        indexVo.setFaultOrderCount(etFaultMapper.getAuditFaultNum(areaId));//待审核故障数量
        return indexVo;
    }

    /**
     * 订单信息统计
     */
    private IndexVo.OrderFeeStatisticsVo getOrderFeeStatistics(String startDateStr, String endDateStr, Long areaId) {
        IndexVo.OrderFeeStatisticsVo feeStatisticsVo = new IndexVo.OrderFeeStatisticsVo();
        BigDecimal payFee = defaultIfNull(etOrderMapper.getPayFee(startDateStr, endDateStr, null, areaId), BigDecimal.ZERO);//新增
        BigDecimal refundFee = defaultIfNull(etOrderMapper.getRefundFee(startDateStr, endDateStr, null, areaId), BigDecimal.ZERO);//退款
        BigDecimal serviceFee = etCapitalFlowMapper.getHandlingFee(startDateStr, endDateStr, null, areaId);//手续费,扣除掉退款部分的
        BigDecimal platformServiceFee = etCapitalFlowMapper.getServiceFee(startDateStr, endDateStr, null,areaId);//平台服务费 ,扣除掉退款部分的
        BigDecimal incomeFee = defaultIfNull(payFee.subtract(serviceFee).subtract(refundFee).subtract(platformServiceFee), BigDecimal.ZERO);//营收 = 新增 - 手续费 - 退款 - 平台服务费
        int orderCount = etOrderMapper.getOrderNum(startDateStr, endDateStr, areaId);
        feeStatisticsVo.setOrderFee(payFee);
        feeStatisticsVo.setIncomeFee(incomeFee);
        feeStatisticsVo.setOrderCount(orderCount);
        return feeStatisticsVo;
    }

    /**
     * 首页统计-总管理
     */
    @Override
    public IndexAdminVo admimStatistics() {
        /** 首页数据*/
        IndexAdminVo indexAdminVo = createIndexAdminVo();

        /** 营收统计*/
        ArrayList<IndexAdminVo.IncomeVo> incomeVos = new ArrayList<>();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.getNowDate());

        for (int i = 0; i < 13; i++) {
            String formattedDate = dateFormat.format(calendar.getTime());
            String startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
            String endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;
            int orderCount = etOrderMapper.getOrderNum(startDateStr, endDateStr, null);
            IndexAdminVo.IncomeVo incomeVo = new IndexAdminVo.IncomeVo();
            incomeVo.setDay(formattedDate);
            incomeVo.setOrderNum(orderCount);
            incomeVo.setOrderFee(etOrderMapper.getOrderFee(startDateStr, endDateStr, null));
            incomeVos.add(incomeVo);
            calendar.add(Calendar.DATE, -1);
        }
        indexAdminVo.setIncomeVoList(incomeVos);

        /** 车辆统计*/
        IndexAdminVo.VehicleVo vehicleVo = new IndexAdminVo.VehicleVo();
        DeviceNumVo deviceNumVo = asDeviceService.allVehicleNum("0","100",null);
        com.ruoyi.common.utils.bean.BeanUtils.copyBeanProp(vehicleVo,deviceNumVo);
        indexAdminVo.setVehicleVo(vehicleVo);

        /** 运维统计*/
        String formattedDate = dateFormat.format(calendar.getTime());
        String todayStartDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
        String todayEndDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;

        IndexAdminVo.OperationVo operationVo = new IndexAdminVo.OperationVo();
        operationVo.setReplacementOrderCount(etAdminOrderMapper.getOrderCount(ServiceConstants.ADMIN_ORDER_TYPE_REPLACEMENT,ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_UNDER_WAY,null));//待换电
        operationVo.setRepairCount(etAdminOrderMapper.getOrderCount(ServiceConstants.ADMIN_ORDER_TYPE_REPAIR,ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_REPAIR,null));//待维修
        operationVo.setFailedUnlockCount(operLogService.getUnlockCount(todayStartDateStr, todayEndDateStr,BusinessStatus.FAIL.name()));// 4g解锁失败
        operationVo.setUnlockCount(operLogService.getUnlockCount(todayStartDateStr, todayEndDateStr,BusinessStatus.SUCCESS.name()));//4g解锁次数
        operationVo.setDeviceOfflineRate(new BigDecimal(deviceNumVo.getOfflineNum()).divide(new BigDecimal(deviceNumVo.getInOperation()), 2, RoundingMode.HALF_UP));//设备离线率
        operationVo.setBluetoothUnlockCount(operLogService.getBluetoothUnlockCount(todayStartDateStr, todayEndDateStr,BusinessStatus.SUCCESS.name()));//蓝牙解锁数
        indexAdminVo.setOperationVo(operationVo);
        log.info("【首页统计-总管理】indexAdminVo==={}",JSON.toJSON(indexAdminVo));

        return indexAdminVo;
    }

    /**
     * 首页统计-总管理
     */
    @Override
    public List<LeaderboardVo> leaderboard(String type,String timeLimit) {
        List<EtOperatingArea> etOperatingAreas = etOperatingAreaService.selectAreaAll();
        ArrayList<LeaderboardVo> leaderboardVos = new ArrayList<>();

        String startDateStr = null;
        String endDateStr = null;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.getNowDate());
        if("1".equals(timeLimit)) {//1-今日
            String formattedDate = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
            startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
            endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;
        }else if("2".equals(timeLimit)){// 一周
            // 设置结束日期为现在
            endDateStr = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime()) + " " + Constants.DATE_FORMAT_END_PEREND;
            // 设置开始日期为7天前
            calendar.add(Calendar.DAY_OF_YEAR, -7);
            startDateStr = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime()) + " " + Constants.DATE_FORMAT_START_PEREND;
        }
        for (EtOperatingArea area : etOperatingAreas) {
            Long areaId = area.getAreaId();
            LeaderboardVo leaderboardVo = new LeaderboardVo();
            leaderboardVo.setOrderCount(etOrderMapper.getOrderNum(startDateStr, endDateStr, areaId));//订单数
            BigDecimal payFee = etOrderMapper.getPayFee(startDateStr, endDateStr, null, areaId);
            BigDecimal refundFee = etOrderMapper.getRefundFee(startDateStr, endDateStr, null, areaId);
            leaderboardVo.setOrderFee(payFee.subtract(refundFee));//订单金额
            String inOrderCount = etOrderMapper.getRidingOrder(startDateStr, endDateStr,areaId+"");
            leaderboardVo.setInProgressOrderCount(Integer.parseInt(inOrderCount));
            leaderboardVo.setAreaId(areaId);
            leaderboardVo.setAreaName(area.getAreaName());
            leaderboardVos.add(leaderboardVo);
        }
        if("1".equals(type)){//1-根据订单数排序
            Collections.sort(leaderboardVos, new Comparator<LeaderboardVo>() {
                @Override
                public int compare(LeaderboardVo o1, LeaderboardVo o2) {
                    // 降序排列，订单数最多的排在最前面
                    return o2.getOrderCount().compareTo(o1.getOrderCount());
                }
            });
        }else if("2".equals(type)){// 2-根据订单金额排序
            Collections.sort(leaderboardVos, new Comparator<LeaderboardVo>() {
                @Override
                public int compare(LeaderboardVo o1, LeaderboardVo o2) {
                    // 降序排列，订单金额最多的排在最前面
                    return o2.getOrderFee().compareTo(o1.getOrderFee());
                }
            });
        }else if("3".equals(type)){// 3-进行中的订单排序
            Collections.sort(leaderboardVos, new Comparator<LeaderboardVo>() {
                @Override
                public int compare(LeaderboardVo o1, LeaderboardVo o2) {
                    // 降序排列，订单金额最多的排在最前面
                    return o2.getInProgressOrderCount().compareTo(o1.getInProgressOrderCount());
                }
            });
        }else{
            throw new RuntimeException("type参数错误");
        }
        // 保留前20个元素
        if (leaderboardVos.size() > 20) {
            leaderboardVos = new ArrayList<>(leaderboardVos.subList(0, 20));
        }
        return leaderboardVos;
    }

    private IndexAdminVo createIndexAdminVo() {
        IndexAdminVo indexAdminVo = new IndexAdminVo();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.getNowDate());
        String formattedDate = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
        String startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
        String endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;

        indexAdminVo.setOperatorCount(deptService.selectAllCount());// 运营商数
        indexAdminVo.setAppCount(deptService.getAppCount());// 独立小程序

        indexAdminVo.setTotalBalance(deptService.selectAllBalance());//商户总余额
        BigDecimal withdrawFee = etCapitalFlowService.getWithdrawnFee(null,ServiceConstants.FLOW_STATUS_COMPLETE);
        indexAdminVo.setWithdrawn(defaultIfNull(withdrawFee, BigDecimal.ZERO));//已提现金额

        indexAdminVo.setAreaCount(etOperatingAreaService.count());//运营区数
        indexAdminVo.setModelCount(etModelService.selectAllCount());//车辆型号数
        indexAdminVo.setUserCount(asUserMapper.selectAllCount(null,null));//用户总数
        indexAdminVo.setTodayUserCount(asUserMapper.selectAllCount(startDateStr,endDateStr));//今日新增

        indexAdminVo.setTodayOrderCount(etOrderMapper.getOrderNum(startDateStr, endDateStr, null));//今日订单数
        indexAdminVo.setTodayOrderFee(etOrderMapper.getPayFee(startDateStr, endDateStr, null, null));//今日订单金额

        indexAdminVo.setTotalOrderCount(etOrderMapper.getOrderNum(null, null, null));//总订单数
        indexAdminVo.setTotalOrderFee(etOrderMapper.getPayFee(null, null, null, null));//总订单金额

        indexAdminVo.setTodayRefundFee(new BigDecimal(etOrderMapper.getTotalRefund(startDateStr,endDateStr,null)));// 今日退款金额
        indexAdminVo.setTotalRefundFee(new BigDecimal(etOrderMapper.getTotalRefund(null,null,null)));// 总退款金额

        indexAdminVo.setUnpaidOrderCount(Integer.parseInt(etOrderMapper.getUnpaidOrder(null,null,null)));//待付款订单数
        indexAdminVo.setUnpaidOrderFee(new BigDecimal(etOrderMapper.getTotalUnpaid(null,null,null)));//待付款订单金额

        indexAdminVo.setReturnOrderCount(etOrderMapper.getAuditOrderNum(null,null,null));//还车待审核订单
        indexAdminVo.setReturnOrderDeductFee(etOrderMapper.getReturnOrderDeductFee(null,null,null));// 待审核还车押金扣款

        indexAdminVo.setTodayHandlingFee(etCapitalFlowMapper.getHandlingFee(startDateStr, endDateStr, null,null));// 今日支付手续费
        indexAdminVo.setTotalHandlingFee(etCapitalFlowMapper.getHandlingFee(null, null, null,null));// 总手续费

        indexAdminVo.setTodayServiceFee(etCapitalFlowMapper.getServiceFee(startDateStr, endDateStr, null,null));//今日服务费
        indexAdminVo.setTotalServiceFee(etCapitalFlowMapper.getServiceFee(null, null, null,null));// 总服务费

        return indexAdminVo;
    }

    private boolean toBePaid(String[] statusList) {
        boolean hasOne = false;
        boolean hasThree = false;
        for (String status : statusList) {
            if ("1".equals(status)) {
                hasOne = true;
            } else if ("3".equals(status)) {
                hasThree = true;
            }
            // 如果已经找到了1和3，无需继续遍历
            if (hasOne && hasThree) {
                break;
            }
        }
        // 返回是否同时包含1和3
        return hasOne && hasThree;
    }

    private boolean ended(String[] statusList) {
        // 长度为1表示只有一个元素
        if (statusList.length != 1) {
            return false;
        }
        // 检查唯一元素的值是否为4
        return "4".equals(statusList[0]);
    }


    /**
     * 新增订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    @Override
    public int insertEtOrder(EtOrder etOrder)
    {
        etOrder.setCreateTime(DateUtils.getNowDate());
        return etOrderMapper.insertEtOrder(etOrder);
    }

    /**
     * 修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    @Override
    public int updateEtOrder(EtOrder etOrder)
    {
        return etOrderMapper.updateEtOrder(etOrder);
    }

    /**
     * 根据订单号修改订单
     *
     * @param etOrder 订单
     * @return 结果
     */
    @Override
    public int updateEtOrderByOrderNo(EtOrder etOrder)
    {
        return etOrderMapper.updateEtOrderByOrderNo(etOrder);
    }

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteEtOrderByOrderIds(Long[] orderIds)
    {
        return etOrderMapper.deleteEtOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteEtOrderByOrderId(Long orderId)
    {
        return etOrderMapper.deleteEtOrderByOrderId(orderId);
    }

    /**
     * 预下单
     * 类型 1骑行 2预约 4押金
     * 获取到订单信息后，计算金额，调用微信支付接口，返回预支付信息
     */
    @Transactional
    @SneakyThrows
    @Override
    public PrepayWithRequestPaymentResponse preOrder(@RequestBody EtOrderVo order) {
        String payType = order.getType();
        if(ObjectUtils.isEmpty(payType)){
            throw new ServiceException("订单类型不能为空");
        }
        //这里的订单号只有押金订单才需要
        EtOrder etOrder = null;
        //根据订单号查订单
        if(payType.equals("4")){
            log.info("【预下单】支付场景为：押金支付");
            // TODO 是否存在押金订单，已经存在则不需要充值
            if(isExitDepositOrder(order.getUserId())){
                throw new ServiceException("您已经充值押金，请勿重复充值");
            }
            String orderNo = IdUtils.getOrderNo("yj");
            // 押金订单，直接生成订单
            etOrder = createOrder(order, orderNo);
            int i = etOrderMapper.insertEtOrder(etOrder);
            if(i == 0){
                throw new ServiceException("订单生成失败");
            }
        }else if(payType.equals("2")){
            log.info("【预下单】支付场景为：取消预约支付");
            etOrder = etOrderMapper.selectEtOrderByOrderNo(order.getOrderNo());
            etOrder.setTotalFee(etOrder.getAppointmentFee());
            etOrder.setPayFee(etOrder.getAppointmentFee());
            int updateEtOrder = etOrderService.updateEtOrder(etOrder);
            if(updateEtOrder==0){
                throw new ServiceException("【预下单】订单信息更新失败");
            }
        }else{
            log.info("【预下单】支付场景为：骑行支付");
            etOrder = etOrderMapper.selectEtOrderByOrderNo(order.getOrderNo());
            if(!ServiceConstants.ORDER_STATUS_RIDING_END.equals(etOrder.getStatus())){
                throw new ServiceException("【预下单】该订单非‘待支付’状态，状态："+etOrder.getStatus()+"，请刷新页面！");
            }
        }
        //TODO 计算订单金额，更新订单金额  还要根据车辆定位，
        // 判断是否在停车区内，
        //      如果在停车区，正常结算费用（根据距离和距离收费，计算费用）
        //      如果不在停车区，加上停车点外调度费，
        // 再判断是否在运营区内
        //      如果在运营区内，
        //      如果不在运营区内，加上调度费
        // 根据下单时间后24小时内封顶200元  完成
        //
        //TODO 订单距离根据定位计算，订单时长根据时间计算
//        etOrder.setDistance(order.getDistance());//距离：米
//        etOrder.setDuration(order.getDuration());//订单时长：以秒为单位

        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(payType,etOrder);
        PrepayWithRequestPaymentResponseVo responseVo = new PrepayWithRequestPaymentResponseVo();
        BeanUtils.copyProperties(response,responseVo);
        responseVo.setSessionId(IdUtils.randomUUIDByDigit(8));
        responseVo.setOrderNo(etOrder.getOrderNo());
        return responseVo;
    }

    /**
     * 是否已充值过押金
     * @param userId
     * @return
     */
    private boolean isExitDepositOrder(Long userId) {
        AsUser asUser = asUserService.selectUserById(userId);
        return asUser.getBalance().compareTo(BigDecimal.ZERO)!=0;
    }

    @Override
    public EtOrder createOrder(EtOrderVo order,String orderNo) {
        AsUser user = asUserService.selectUserById(order.getUserId());
        EtOrder etOrder = new EtOrder();
        etOrder.setOrderNo(orderNo);
        etOrder.setUserId(user.getUserId());
        etOrder.setUserName(user.getUserName());
        etOrder.setMark(order.getMark());
        etOrder.setCreateTime(DateUtils.getNowDate());
        if(ObjectUtil.isNotNull(order.getRuleId())){
            EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleIdIncludeDelete(order.getRuleId());
            etOrder.setAutoRefundDeposit(etFeeRule.getAutoRefundDeposit());
            etOrder.setFreeRideTime(etFeeRule.getFreeRideTime());
            etOrder.setRentalUnit(etFeeRule.getRentalUnit());
            etOrder.setRidingRule(etFeeRule.getRidingRule());
            etOrder.setRidingRuleJson(etFeeRule.getRidingRuleJson());
            etOrder.setChargingCycle(etFeeRule.getChargingCycle());
            etOrder.setChargingCycleValue(etFeeRule.getChargingCycleValue());
            etOrder.setCappedAmount(etFeeRule.getCappedAmount());
        }
        etOrder.setRuleId(order.getRuleId());
        etOrder.setAreaId(order.getAreaId());
        /** 订单类型 1骑行 2预约 3套餐 4押金 根据类型不同下单的参数也不同，不同的参数有： 金额、类型、rule_id（套餐id）*/
        String type = order.getType();
        if(type.equals("1")){//骑行订单,正常骑行（包含预约费），计时收费，根据开锁时间、起步价和起步时长、时长费和时长分钟等参数，如果有预约，还要加上预约费计算费用
            setAreaInfo(order.getSn(), etOrder);
            etOrder.setType(ServiceConstants.ORDER_TYPE_RIDING);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
            etOrder.setUnlockTime(DateUtils.getNowDate());
        }else if(order.getType().equals("2")){//预约，创建初始订单，根据开始预约时间结算费用，开始骑行时，结算预约费用，在订单表中通过appointment_start_time字段是否有值或者status是1（取消预约）和paid=1，来判断是否是预约订单
            setAreaInfo(order.getSn(), etOrder);
            etOrder.setType(ServiceConstants.ORDER_TYPE_RIDING);
            etOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_APPOINTMENT);//预约中
            etOrder.setAppointmentStartTime(DateUtils.getNowDate());//预约开始时间
        }else if(order.getType().equals("4")){//押金，创建初始订单，根据计费规则，结算费用，
            etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
            if(ObjectUtil.isNull(order.getAreaId())){
                log.info("【预下单-创建订单】支付场景为：押金支付--order.getAreaId()"+order.getAreaId());
                throw new ServiceException("【预下单-创建订单】支付场景为：押金支付--order.getAreaId()"+order.getAreaId());
            }
            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
            BigDecimal deposit = new BigDecimal(area.getDeposit());
            etOrder.setTotalFee(deposit);
            etOrder.setPayFee(deposit);
        }else {
            throw new ServiceException("订单类型错误");
        }
        return etOrder;
    }

    private void verify(EtOrderVo order, Long userId) {
        if(etOrderService.isInOrder(userId,null).size()>0){
           throw new ServiceException("您有未完成的订单，请先完成订单");
        }
        //根据余额和充值记录判断是否有充值过押金，没有充值过押金，提示充值押金
        if(!asUserService.checkIsDeposit(userId)){
            throw new ServiceException("您还未充值押金，请先充值押金");
        }
        //检验预约订单只能有一个
        if(etOrderService.checkIsUnique(userId)){
            throw new ServiceException("您有预约中的订单，请务重复操作");
        }
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(order.getSn());
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_NORMAL.equals(status)){
            //根据状态值返回不同的提示
            throw new ServiceException(CommonUtil.format(status));
        }
    }

    /**
     * 检验预约订单只能有一个
     */
    @Override
    public Boolean checkIsUnique(Long userId) {
        int i = etOrderMapper.checkIsUnique(userId);
        return i>0;
    }


    /**
     * 运营数据
     */
    @Override
    public OperatingDataVo getOperatingData(String timeStart,String timeEnd,String areaId) {
        OperatingDataVo operatingDataVo = new OperatingDataVo();
        if(StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)){
            /*收入相关*/
            OperatingDataVo.Income income = new OperatingDataVo.Income();
            String totalUnpaid = etOrderMapper.getTotalUnpaid(timeStart, timeEnd, areaId);
            income.setTotalUnpaid(totalUnpaid);//累计待支付
            String totalPaid = etOrderMapper.getTotalPaid(timeStart, timeEnd, areaId);
            String totalRefund = etOrderMapper.getTotalRefund(timeStart, timeEnd, areaId);//已退款
            income.setTotalPaid(totalPaid);//已支付
            //handlingFee 手续费 = 0.0054 * 已支付金额
            BigDecimal handlingFee = etCapitalFlowMapper.getHandlingFee(timeStart, timeEnd, null, Long.parseLong(areaId));//手续费,扣除掉退款部分的
            BigDecimal platformServiceFee = etCapitalFlowMapper.getServiceFee(timeStart, timeEnd, null,Long.parseLong(areaId));//平台服务费 ,扣除掉退款部分的
            income.setHandlingFee(handlingFee.add(platformServiceFee).toString());

            // 总营收 = 已支付 - 已退款 - 手续费
            income.setTotalIncome(new BigDecimal(totalPaid).subtract(new BigDecimal(totalRefund)).subtract(handlingFee).subtract(platformServiceFee).toString());//总收入
            income.setTotalRefund(totalRefund);//已退款
            income.setTotalFee(new BigDecimal(totalUnpaid).add(new BigDecimal(totalPaid)).toString());// 订单总金额 = 已支付+待支付
            income.setTotalRidingFee(etOrderMapper.getTotalRidingFee(timeStart,timeEnd,areaId));//骑行已支付
            income.setTotalRidingRefund(etOrderMapper.getTotalRidingRefund(timeStart,timeEnd,areaId));//骑行已退款
            income.setTotalDispatchFee(etOrderMapper.getTotalDispatchFee(timeStart,timeEnd,areaId));//调度费已支付
            income.setTotalDispatchRefund(etOrderMapper.getTotalDispatchRefund(timeStart,timeEnd,areaId));//调度费已退款
            income.setTotalAppointmentFee(etOrderMapper.getTotalAppointmentFee(timeStart,timeEnd,areaId));//预约费已支付
            income.setTotalAppointmentRefund(etOrderMapper.getTotalAppointmentRefund(timeStart,timeEnd,areaId));//预约费已退款
            income.setTotalManageFee(etOrderMapper.getTotalManageFee(timeStart,timeEnd,areaId));//停车点外调度费已支付
            income.setTotalManageRefund(etOrderMapper.getTotalManageRefund(timeStart,timeEnd,areaId));//停车点外调度费已退款
            operatingDataVo.setIncome(income);

            /*订单相关*/
            OperatingDataVo.OrderVo orderVo = new OperatingDataVo.OrderVo();
            orderVo.setPaidOrder(etOrderMapper.getPaidOrder(timeStart,timeEnd,areaId));//已支付订单
            orderVo.setRidingOrder(etOrderMapper.getRidingOrder(timeStart,timeEnd,areaId));//骑行中订单
            orderVo.setRefundOrder(etOrderMapper.getRefundOrder(timeStart,timeEnd,areaId));//退款订单
            orderVo.setUnpaidOrder(etOrderMapper.getUnpaidOrder(timeStart,timeEnd,areaId));//待支付订单
            operatingDataVo.setOrder(orderVo);

            /*设备相关*/
            OperatingDataVo.DeviceVo deviceVo = new OperatingDataVo.DeviceVo();
            String inOperationDevice = deviceService.getInOperationDevice(timeStart, timeEnd, areaId);
            String inOrderDevice = deviceService.getInOrderDevice(timeStart, timeEnd, areaId);
            deviceVo.setInOperationDevice(inOperationDevice);//运营中车辆
            deviceVo.setInOrderDevice(inOrderDevice);//有订单车辆
            int i = Integer.parseInt(inOperationDevice) - Integer.parseInt(inOrderDevice);
            deviceVo.setNoOrderDevice( i+"");//无订单车辆
            operatingDataVo.setDevice(deviceVo);

            /*用户相关*/
            OperatingDataVo.UserVo userVo = new OperatingDataVo.UserVo();
            userVo.setTotalUser(asUserService.getTotalUser(areaId));//总用户
            userVo.setNewUser(asUserService.getNewUser(timeStart,timeEnd,areaId));//新增用户
            userVo.setLeaseUser(asUserService.getLeaseUser(timeStart,timeEnd,areaId));//租赁用户
            operatingDataVo.setUser(userVo);
            return operatingDataVo;
        }else{
            throw new ServiceException("请选择时间");
        }
    }

    /**
     * 运营数据
     */
    @Override
    public OperatingDataVo2 getOperatingData2(String timeStart,String timeEnd,String areaId) {
        long aLong = 0;
        if(StrUtil.isNotBlank(areaId)){
            aLong = Long.parseLong(areaId);
        }
        OperatingDataVo2 operatingDataVo = new OperatingDataVo2();
        if(StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)){
            /*收入相关*/
            OperatingDataVo2.Income income = new OperatingDataVo2.Income();
            /** 总收入*/
            BigDecimal totalAmount = defaultIfNull(etCapitalFlowMapper.getTotalAmount(timeStart, timeEnd, null, aLong),BigDecimal.ZERO);// 骑行订单收入
            BigDecimal deductionAmount = defaultIfNull(etCapitalFlowMapper.getDeductionAmount(timeStart, timeEnd, null, aLong),BigDecimal.ZERO);// 押金抵扣金额
            BigDecimal depositAmount = defaultIfNull(etOrderMapper.getDepositAmount(timeStart, timeEnd,  aLong,null),BigDecimal.ZERO);// 押金收入  25123
            BigDecimal totalFlowAmount = totalAmount.add(depositAmount).add(deductionAmount);// 总流水
            income.setTotalFlowAmount(totalFlowAmount);
            income.setDeductionAmount(deductionAmount);
            income.setOrderPaid(totalAmount.add(deductionAmount).add(deductionAmount));//订单支付
            income.setDepositPaid(depositAmount);

            /** 总支出*/
            BigDecimal refundFee = defaultIfNull(etOrderMapper.getRefundFee(timeStart, timeEnd, null, aLong), BigDecimal.ZERO);//订单退款
            BigDecimal depositRefundFee = defaultIfNull(etOrderMapper.getDepositRefundFee(timeStart, timeEnd, null, aLong,null), BigDecimal.ZERO);//押金退款 24795
            BigDecimal serviceFee = defaultIfNull(etCapitalFlowMapper.getHandlingFee(timeStart, timeEnd, null, aLong),BigDecimal.ZERO);//手续费,扣除掉退款部分的
            BigDecimal platformServiceFee = defaultIfNull(etCapitalFlowMapper.getServiceFee(timeStart, timeEnd, null,aLong),BigDecimal.ZERO);//平台服务费 ,扣除掉退款部分的
            BigDecimal totalExpenditure = defaultIfNull(refundFee.add(depositRefundFee).add(serviceFee).add(platformServiceFee), BigDecimal.ZERO);

            income.setOrderRefund(refundFee);
            income.setDepositRefund(depositRefundFee);
            income.setHandlingFee(serviceFee);
            income.setPlatformServiceFee(platformServiceFee);
            income.setTotalExpenditure(totalExpenditure);

            operatingDataVo.setIncome(income);
            BigDecimal depositBalance = depositAmount.subtract(depositRefundFee).subtract(deductionAmount);
            income.setOperatingIncome(totalFlowAmount.subtract(totalExpenditure).subtract(depositBalance));
            income.setOperatingIncome2(totalAmount.add(deductionAmount).subtract(refundFee).subtract(serviceFee).subtract(platformServiceFee).add(deductionAmount));
            BigDecimal depositHandlingCharge = depositBalance.multiply(new BigDecimal(0.0054));// 押金手续费
            depositHandlingCharge = depositHandlingCharge.setScale(2, RoundingMode.HALF_UP);
            income.setDepositBalance(depositBalance.subtract(depositHandlingCharge));// 押金余额
            income.setDepositHandlingCharge(depositHandlingCharge);// 押金手续费
            income.setAccountSurplus(depositBalance.add(totalFlowAmount.subtract(totalExpenditure).subtract(depositBalance)).subtract(depositHandlingCharge));// 账户盈余=总流水-总支出 = 营业收入+押金余额  depositBalance+operatingIncome
            BigDecimal businessCost = defaultIfNull(refundFee.add(serviceFee).add(platformServiceFee), BigDecimal.ZERO);//主营业务成本
            income.setBusinessCost(businessCost);

            /*订单相关*/
            OperatingDataVo2.OrderVo orderVo = new OperatingDataVo2.OrderVo();
            orderVo.setPaidOrder(etOrderMapper.getPaidOrder(timeStart,timeEnd,areaId));//已支付订单
            orderVo.setRidingOrder(etOrderMapper.getRidingOrder(timeStart,timeEnd,areaId));//骑行中订单
            orderVo.setRefundOrder(etOrderMapper.getRefundOrder(timeStart,timeEnd,areaId));//退款订单
            orderVo.setUnpaidOrder(etOrderMapper.getUnpaidOrder(timeStart,timeEnd,areaId));//待支付订单
            operatingDataVo.setOrder(orderVo);
            return operatingDataVo;
        }else{
            throw new ServiceException("请选择时间");
        }
    }

//    /**
//     * 收入对账
//     * type:1-按日期 2-按车辆
//     */
//    @SneakyThrows
//    @Override
//    public ReconciliationVo reconciliation(String timeStart, String timeEnd, String type,String sn) {
//        ReconciliationVo reconciliationVo = new ReconciliationVo();
//        List<ReconciliationVo.Reconciliation> reconciliations = new ArrayList<>();
//        if("1".equals(type)){//按日期
//            // 格式化日期
//            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
//            int page = 1;
//            int limit = DateUtils.differentDaysByMillisecond(timeStart, timeEnd)+1;
//
//            // 获取当前日期
//            Calendar calendar = Calendar.getInstance();
//            Date date1 = dateFormat.parse(timeEnd);
//            calendar.setTime(date1);
//
//            // 计算起始日期
//            calendar.add(Calendar.DATE, -(page - 1) * limit);
//            Date startDate = calendar.getTime();
//
//            LoginUser loginUser = SecurityUtils.getLoginUser();
//            SysUser user = loginUser.getUser();
//            Long deptId = user.getDeptId();
//            List<Long> longs ;
//            if(deptId == 100){//如果是创享管理员则展示所有数据
//                longs = etOperatingAreaService.selectAreaListByDeptId(null);
//            }else{
//                longs = etOperatingAreaService.selectAreaListByDeptId(deptId);
//            }
//            for (int i = 0; i < limit; i++) {
//                for(Long areaId : longs){
//                    String formattedDate = dateFormat.format(startDate);
//                    String startDateStr = formattedDate + " "+ Constants.DATE_FORMAT_START_PEREND;
//                    String endDateStr = formattedDate + " " +Constants.DATE_FORMAT_END_PEREND;
//                    if(StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)){
//                        ReconciliationVo.Reconciliation reconciliation = new ReconciliationVo.Reconciliation();
//                        reconciliation.setAreaName(etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId).getAreaName());
//                        reconciliation.setDeptName(deptService.selectDeptById(deptId).getDeptName());
//                        //获取当前日期的订单
//                        reconciliation.setDay(formattedDate);
//                        BigDecimal payFee = etOrderMapper.getPayFee(startDateStr, endDateStr, null,areaId);// 新增
//                        BigDecimal defaultPayFee = payFee != null ? payFee : BigDecimal.ZERO;
//                        reconciliation.setPayFee(defaultPayFee);
//                        BigDecimal refundFee = etOrderMapper.getRefundFee(startDateStr, endDateStr, null, areaId);// 退款
//                        BigDecimal defaultRefundFee = refundFee != null ? refundFee : BigDecimal.ZERO;
//                        reconciliation.setRefundFee(defaultRefundFee);
//                        BigDecimal incomeFee = etOrderMapper.getIncome(startDateStr, endDateStr, null); // 营收
//                        BigDecimal defaultIncomeFeeFee = incomeFee != null ? incomeFee : BigDecimal.ZERO;
//                        reconciliation.setIncome(defaultIncomeFeeFee);
//                        BigDecimal serviceFee = defaultPayFee.multiply(new BigDecimal(handlingCharge).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP));
//                        reconciliation.setServiceFee(serviceFee);// 手续费
//
//                        // 合伙人分账
//                        BigDecimal partnerFee = etOrderMapper.getPartnerFee(startDateStr, endDateStr, areaId);
//                        reconciliation.setPartnerFee(partnerFee);
//                        // 运营商分账
//                        BigDecimal platformServiceFee = etOrderMapper.getPlatformServiceFee(startDateStr, endDateStr, areaId);
//                        reconciliation.setPlatformServiceFee(platformServiceFee);
//                        reconciliations.add(reconciliation);
//                    }
//                    // 日期减一天
//                    calendar.add(Calendar.DATE,-1);
//                    startDate = calendar.getTime();
//                }
//            }
//            reconciliationVo.setReconciliationList(reconciliations);
//            //计算总收入   payFee
//            // 使用Java 8 Stream API来计算payFee字段的总和
//            BigDecimal totalPayFee = reconciliations != null && !reconciliations.isEmpty()
//                    ? reconciliations.stream()
//                    .map(ReconciliationVo.Reconciliation::getPayFee)
//                    .reduce(BigDecimal.ZERO, BigDecimal::add)
//                    : null;
//            reconciliationVo.setTotalPayFee(totalPayFee);
//
//            BigDecimal totalRefundFee = reconciliations != null && !reconciliations.isEmpty()
//                    ? reconciliations.stream()
//                    .map(ReconciliationVo.Reconciliation::getRefundFee)
//                    .reduce(BigDecimal.ZERO, BigDecimal::add)
//                    : null;
//            reconciliationVo.setTotalRefundFee(totalRefundFee);
//
//            BigDecimal totalServiceFee = reconciliations != null && !reconciliations.isEmpty()
//                    ? reconciliations.stream()
//                    .map(ReconciliationVo.Reconciliation::getServiceFee)
//                    .reduce(BigDecimal.ZERO, BigDecimal::add)
//                    : null;
//            reconciliationVo.setTotalServiceFee(totalServiceFee);
//
//            BigDecimal totalIncome = reconciliations != null && !reconciliations.isEmpty()
//                    ? reconciliations.stream()
//                    .map(ReconciliationVo.Reconciliation::getIncome)
//                    .reduce(BigDecimal.ZERO, BigDecimal::add)
//                    : null;
//            reconciliationVo.setTotalIncome(totalIncome);
//
//            return reconciliationVo;
//        }else {
//            if(StrUtil.isBlank(sn)){
//               throw new ServiceException("类型为2时，车辆的sn必传");
//            }
//            AsDevice asDevice = new AsDevice();
//            asDevice.setSn(sn);
//            // 按车辆,先把所有车辆查出，如果参数中有车辆，就查出该车辆的按日期的订单
//            List<AsDevice> asDevices = asDeviceService.selectAsDeviceList(asDevice);
//            for(AsDevice asDevice1:asDevices){
//                String sn1 = asDevice1.getSn();
//                if(StrUtil.isNotBlank(sn1)){
//                    //获取当前日期的订单
//                    ReconciliationVo.Reconciliation reconciliation = new ReconciliationVo.Reconciliation();
//                    reconciliation.setSn(sn1);
//                    reconciliation.setPayFee(etOrderMapper.getPayFee(timeStart,timeEnd,sn1,null));
//                    reconciliation.setRefundFee(etOrderMapper.getRefundFee(timeStart,timeEnd,sn1,null));
//                    reconciliation.setIncome(etOrderMapper.getIncome(timeStart,timeEnd,sn1));
//                    reconciliations.add(reconciliation);
//                }
//            }
//            reconciliationVo.setReconciliationList(reconciliations);
//            return reconciliationVo;
//        }
//    }

    /**
     * 收入对账
     * type: 1-按日期 2-按车辆
     */
    @SneakyThrows
    public ReconciliationVo reconciliation(String timeStart, String timeEnd, String type, String sn,String areaId) {
        ReconciliationVo reconciliationVo = new ReconciliationVo();
        List<ReconciliationVo.Reconciliation> reconciliations = new ArrayList<>();

        if ("1".equals(type)) {
            reconciliationVo = handleReconciliationByDate(timeStart, timeEnd, reconciliations,areaId);
        } else if ("2".equals(type)) {
            reconciliationVo = handleReconciliationByVehicle(timeStart, timeEnd, sn, reconciliations);
        } else {
            throw new ServiceException("Invalid type");
        }

        return reconciliationVo;
    }

    /**
     * 平台对账
     * */
    @SneakyThrows
    @Override
    public List<SelfReconciliationVO> selfReconciliation(String timeStart, String timeEnd, String areaId) {
        long aLong = 0;
//        if(StrUtil.isNotBlank(areaId)){
//            aLong = Long.parseLong(areaId);
//        }
//        if(StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)){
//            List<SelfReconciliationVO> selfReconciliationVOS = new ArrayList<>();
//            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
//            int limit = DateUtils.differentDaysByMillisecond(timeStart, timeEnd) + 1;
//
//            Calendar calendar = Calendar.getInstance();
//            calendar.setTime(dateFormat.parse(timeEnd));
//
//            for (int i = 0; i < limit; i++) {
//                SelfReconciliationVO selfReconciliationVO = new SelfReconciliationVO();
//                String formattedDate = dateFormat.format(calendar.getTime());
//                String startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
//                String endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;
//
//                /** 总收入*/
//                selfReconciliationVO.setDay(formattedDate);
//                BigDecimal totalFlowAmount = defaultIfNull(etOrderMapper.getTotalPaidFee(startDateStr, endDateStr, aLong),BigDecimal.ZERO);// 骑行订单收入  29835.51
//                BigDecimal totalAmount = defaultIfNull(etCapitalFlowMapper.getOrderPaidAmount(startDateStr, endDateStr, aLong),BigDecimal.ZERO);// 骑行订单收入  4712.51
//                BigDecimal deductionAmount = defaultIfNull(etOrderMapper.getDepositDeductionAmount(startDateStr, endDateStr, aLong),BigDecimal.ZERO);// 押金抵扣金额  538
//                BigDecimal depositAmount = defaultIfNull(etOrderMapper.getDepositAmount(startDateStr, endDateStr,  aLong),BigDecimal.ZERO);// 押金收入  25123
//                BigDecimal handlingFee = etCapitalFlowMapper.getHandlingFee(startDateStr, endDateStr, null, aLong);//手续费,扣除掉退款部分的
//                BigDecimal platformServiceFee = etCapitalFlowMapper.getServiceFee(startDateStr, endDateStr, null,aLong);//平台服务费 ,扣除掉退款部分的
//
//                selfReconciliationVO.setOrderPaid(totalAmount);
//                selfReconciliationVO.setTotalFlowAmount(totalFlowAmount);
//                selfReconciliationVO.setDeductionAmount(deductionAmount);
//                selfReconciliationVO.setDepositPaid(depositAmount);
//                selfReconciliationVO.setHandlingCharge(handlingFee);
//                selfReconciliationVO.setPlatformServiceFee(platformServiceFee);
//
//                /** 总支出*/
//                BigDecimal orderRefund = defaultIfNull(etOrderMapper.getRefundFee(startDateStr, endDateStr, null, aLong), BigDecimal.ZERO);//订单退款
//                BigDecimal depositRefundFee = defaultIfNull(etOrderMapper.getDepositRefundFee(startDateStr, endDateStr, null, aLong), BigDecimal.ZERO);//押金退款 24795   25386
//                // depositChange  等于depositAmount减去deductionAmount减去depositRefundFee
//                selfReconciliationVO.setDepositChange(depositAmount.subtract(deductionAmount).subtract(depositRefundFee));
//                // 统计所有用户今日账变  进账
//                BigDecimal userReceipts = defaultIfNull(etCapitalFlowMapper.getAllUserReceipts(startDateStr, endDateStr, aLong),BigDecimal.ZERO);
//
//                selfReconciliationVO.setOrderRefund(orderRefund);
//                selfReconciliationVO.setDepositRefund(depositRefundFee);
//                BigDecimal depositBalance = depositAmount.subtract(depositRefundFee).subtract(deductionAmount);
//                selfReconciliationVO.setDepositBalance(depositBalance);
//                selfReconciliationVO.setUserReceipts(userReceipts);
//                // 结算金额等于totalFlowAmount减去orderRefund减去depositRefundFee
//                selfReconciliationVO.setSettlementAmount(totalFlowAmount.subtract(orderRefund).subtract(depositRefundFee));
//                calendar.add(Calendar.DATE, -1);
//
//                selfReconciliationVOS.add(selfReconciliationVO);
//            }
//            return selfReconciliationVOS;
//        }else{
//            throw new ServiceException("请选择时间");
//        }
        return null;
    }

    private ReconciliationVo handleReconciliationByDate(String timeStart, String timeEnd, List<ReconciliationVo.Reconciliation> reconciliations,String areaId) throws Exception {
        ReconciliationVo reconciliationVo = new ReconciliationVo();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        int limit = DateUtils.differentDaysByMillisecond(timeStart, timeEnd) + 1;

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateFormat.parse(timeEnd));

        for (int i = 0; i < limit; i++) {
            String formattedDate = dateFormat.format(calendar.getTime());
            String startDateStr = formattedDate + " " + Constants.DATE_FORMAT_START_PEREND;
            String endDateStr = formattedDate + " " + Constants.DATE_FORMAT_END_PEREND;

            if (StrUtil.isNotBlank(timeStart) && StrUtil.isNotBlank(timeEnd)) {
                reconciliations.add(createReconciliationByDate(formattedDate, startDateStr, endDateStr, Long.parseLong(areaId)));
            }

            calendar.add(Calendar.DATE, -1);
        }

        reconciliationVo.setReconciliationList(reconciliations);
        calculateTotal(reconciliations, reconciliationVo);

        return reconciliationVo;
    }

    private ReconciliationVo handleReconciliationByVehicle(String timeStart, String timeEnd, String sn, List<ReconciliationVo.Reconciliation> reconciliations) {
        if (StrUtil.isBlank(sn)) {
            throw new ServiceException("类型为2时，车辆的sn必传");
        }
        AsDevice device1 = new AsDevice();
        device1.setSn(sn);
        List<AsDevice> devices = asDeviceMapper.selectAsDeviceList(device1);
        log.info("【收入对账】根据sn搜索出的devices:{}", JSON.toJSON(devices));
        for (AsDevice device : devices) {
            reconciliations.add(createReconciliationByVehicle(timeStart, timeEnd, device.getSn(),device.getAreaId()));
        }

        ReconciliationVo reconciliationVo = new ReconciliationVo();
        reconciliationVo.setReconciliationList(reconciliations);
        return reconciliationVo;
    }

    private ReconciliationVo.Reconciliation createReconciliationByDate(String formattedDate, String startDateStr, String endDateStr, Long areaId) {
        ReconciliationVo.Reconciliation reconciliation = new ReconciliationVo.Reconciliation();
        StringBuilder ridingFee = new StringBuilder();
        StringBuilder dispatchFee = new StringBuilder();
        reconciliation.setAreaName(etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId).getAreaName());
        SysDept deptObjByAreaId = wxPayService.getDeptObjByAreaId(areaId);
        reconciliation.setDeptName(deptObjByAreaId.getDeptName());
        reconciliation.setDay(formattedDate);
        BigDecimal payFee = defaultIfNull(etOrderMapper.getPayFee(startDateStr, endDateStr, null, areaId), BigDecimal.ZERO);//新增
        BigDecimal refundFee = defaultIfNull(etOrderMapper.getRefundFee(startDateStr, endDateStr, null, areaId), BigDecimal.ZERO);//退款
        BigDecimal serviceFee = etCapitalFlowMapper.getHandlingFee(startDateStr, endDateStr, null, areaId);//手续费,扣除掉退款部分的
        BigDecimal platformServiceFee = etCapitalFlowMapper.getServiceFee(startDateStr, endDateStr, null,areaId);//平台服务费 ,扣除掉退款部分的
        BigDecimal incomeFee = defaultIfNull(payFee.subtract(serviceFee).subtract(refundFee).subtract(platformServiceFee), BigDecimal.ZERO);//营收 = 新增 - 手续费 - 退款 - 平台服务费

        reconciliation.setPayFee(payFee);
        reconciliation.setRefundFee(refundFee);
        reconciliation.setIncome(incomeFee);
        reconciliation.setServiceFee(serviceFee);
        reconciliation.setPartnerFee(etOrderMapper.getPartnerFee(startDateStr, endDateStr, areaId));
        reconciliation.setPlatformServiceFee(platformServiceFee);

        ridingFee.append("营收："+incomeFee +"元").append(System.lineSeparator());
        ridingFee.append("新增："+payFee  +"元").append(System.lineSeparator());
        ridingFee.append("退款："+refundFee +"元").append(System.lineSeparator());
        ridingFee.append("手续费："+serviceFee +"元").append(System.lineSeparator());
        ridingFee.append("平台服务费："+platformServiceFee +"元").append(System.lineSeparator());
        reconciliation.setRidingFee(ridingFee.toString());

        dispatchFee.append("营收："+incomeFee +"元").append(System.lineSeparator());
        dispatchFee.append("新增："+payFee  +"元").append(System.lineSeparator());
        dispatchFee.append("退款："+refundFee +"元").append(System.lineSeparator());
        dispatchFee.append("手续费："+serviceFee +"元").append(System.lineSeparator());
        dispatchFee.append("平台服务费："+platformServiceFee +"元").append(System.lineSeparator());
        reconciliation.setDispatchFee(dispatchFee.toString());

        return reconciliation;
    }

    private ReconciliationVo.Reconciliation createReconciliationByVehicle(String timeStart, String timeEnd, String sn,Long areaId) {
        ReconciliationVo.Reconciliation reconciliation = new ReconciliationVo.Reconciliation();
        reconciliation.setSn(sn);
        BigDecimal payFee = etOrderMapper.getPayFee(timeStart, timeEnd, sn, null);
        reconciliation.setPayFee(payFee);
        BigDecimal refundFee = etOrderMapper.getRefundFee(timeStart, timeEnd, sn, null);
        reconciliation.setRefundFee(refundFee);
        BigDecimal serviceFee = etCapitalFlowMapper.getHandlingFee(timeStart, timeEnd, sn, areaId);//手续费
        BigDecimal platformServiceFee = etCapitalFlowMapper.getServiceFee(timeStart, timeEnd, sn, areaId);//平台服务费
        reconciliation.setIncome(payFee.subtract(refundFee).subtract(serviceFee).subtract(platformServiceFee));
        reconciliation.setServiceFee(serviceFee);
        reconciliation.setPlatformServiceFee(platformServiceFee);
        return reconciliation;
    }

//    private List<Long> getAreaIds() {
//        Long deptId = SecurityUtils.getLoginUser().getUser().getDeptId();
//        return deptId == 100 ? etOperatingAreaService.selectAreaListByDeptId(null) : etOperatingAreaService.selectAreaListByDeptId(deptId);
//    }

    private void calculateTotal(List<ReconciliationVo.Reconciliation> reconciliations, ReconciliationVo reconciliationVo) {
        reconciliationVo.setTotalPayFee(calculateTotalField(reconciliations, ReconciliationVo.Reconciliation::getPayFee));
        reconciliationVo.setTotalRefundFee(calculateTotalField(reconciliations, ReconciliationVo.Reconciliation::getRefundFee));
        reconciliationVo.setTotalServiceFee(calculateTotalField(reconciliations, ReconciliationVo.Reconciliation::getServiceFee));
        reconciliationVo.setTotalIncome(calculateTotalField(reconciliations, ReconciliationVo.Reconciliation::getIncome));
        reconciliationVo.setTotalPlatformServiceFee(calculateTotalField(reconciliations, ReconciliationVo.Reconciliation::getPlatformServiceFee));
    }

    private BigDecimal calculateTotalField(List<ReconciliationVo.Reconciliation> reconciliations, java.util.function.Function<ReconciliationVo.Reconciliation, BigDecimal> getter) {
        return reconciliations.stream().map(getter).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    private BigDecimal defaultIfNull(BigDecimal value, BigDecimal defaultValue) {
        return value != null ? value : defaultValue;
    }

    /**
     * 最近一笔订单
     */
    @Override
    public EtOrder latestOrder(EtOrder etOrder) {
        EtOrder order = etOrderMapper.selectLatestOrder(etOrder);
        return order;
    }

    /**
     * 最近一笔订单
     */
    @Override
    public EtOrder latestOrder2(EtOrder etOrder) {
        EtOrder order = etOrderMapper.selectLatestOrder2(etOrder);
        return order;
    }

    /**
     * 退款
     */
    @Transactional
    @Override
    public int refund(EtOrder etOrder) {
        //根据订单号查询订单信息
        EtOrder etOrder1 = etOrderService.selectEtOrderByOrderNo(etOrder.getOrderNo());
        if(ObjectUtil.isNull(etOrder1)){
            throw new ServiceException("订单不存在");
        }
        if(!ServiceConstants.ORDER_TYPE_RIDING.equals(etOrder1.getType())){
            throw new ServiceException("退款失败，类型必须是骑行订单");
        }
        if(!ServiceConstants.ORDER_STATUS_ORDER_END.equals(etOrder1.getStatus())){
            throw new ServiceException("退款失败，订单未结束");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(etOrder1.getPaid())){
            throw new ServiceException("订单未支付，不能退款");
        }
        // 根据分账明细查询
//        List<EtCapitalFlow> etCapitalFlows = etCapitalFlowService.selectEtCapitalFlowByOrderNo(etOrder.getOrderNo());
//        for (EtCapitalFlow etCapitalFlow : etCapitalFlows) {
//            if(etCapitalFlow.getOwnerType().equals("1")){
//                etCapitalFlow.
//            }
//        }
        SysDept sysDept = wxPayService.getDeptObjByAreaId(etOrder1.getAreaId());
        BigDecimal subtract = sysDept.getBalance().subtract(etOrder1.getTotalFee());
        if(subtract.compareTo(BigDecimal.ZERO) <= 0 && sysDept.getSeparateAccount().equals("N")){
            throw new ServiceException("余额不足，不能退款");
        }
        /** 1.退款*/
        //退款金额
        BigDecimal refundAmount = new BigDecimal("0");
        BigDecimal appointmentFee = etOrder.getAppointmentFee();
        if(ObjectUtil.isNotNull(appointmentFee) && !appointmentFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(appointmentFee);
        }
        BigDecimal dispatchFee = etOrder.getDispatchFee();
        if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(dispatchFee);
        }
        BigDecimal manageFee = etOrder.getManageFee();
        if(ObjectUtil.isNotNull(manageFee) && !manageFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(manageFee);
        }
        BigDecimal ridingFee = etOrder.getRidingFee();
        if(ObjectUtil.isNotNull(ridingFee) && !ridingFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(ridingFee);
        }
        // 如果已支付并且支付方式是押金，则退款的流程是根据押金退款，获取押金订单进行退款
        if(etOrder1.getPayType().equals(ServiceConstants.PAY_TYPE_YJ) && etOrder1.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){
            etOrder1 = getDepositOrder(etOrder1.getUserId());
            log.info("【押金抵扣的退款】：【{}】，退款金额：【{}】", etOrder1.getTotalFee(), refundAmount);
            etOrder1.setReason(etOrder.getReason());
        }
        // 新增资金流水记录
        etOrder1.setPayFee(refundAmount);
        callbackService.capitalFlowRecords(etOrder1,ServiceConstants.FLOW_TYPE_DISBURSE,ServiceConstants.ORDER_TYPE_RIDING_REFUND,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_WX);
        //todo 更新订单的payFee = totalFee - refundAmount
        String outRefundNo = IdUtils.getOrderNo("ref");
        /** 2.记录退款表  创建退款对象*/
        EtRefund refund1= createRefund(etOrder1, refundAmount, appointmentFee, dispatchFee, manageFee, ridingFee, outRefundNo,ServiceConstants.REFUND_TYPE_SYSTEM);
        int i = etRefundService.insertEtRefund(refund1);
        if(i>0){
            log.info("保存退款对象成功");
        }
        log.info("总金额：【{}】，退款金额：【{}】", etOrder1.getTotalFee(), refundAmount);
        wxPayService.refund(etOrder1, etOrder.getReason(), refundAmount,outRefundNo);
        return i;
    }

    /**
     * 提现
     */
    @Transactional
    @Override
    public int withdraw(EtOrder etOrder1, BigDecimal deposit) {
        String outRefundNo = IdUtils.getOrderNo("ref");
        /** 2.记录退款表  创建退款对象*/
        etOrder1.setReason("押金提现");
        EtRefund refund1= createRefund(etOrder1, deposit, null, null, null, null, outRefundNo,ServiceConstants.REFUND_TYPE_DEPOSIT);
        int i = etRefundService.insertEtRefund(refund1);
        if(i>0){
            log.info("【提现金额】保存退款对象成功");
        }
        wxPayService.refund(etOrder1, "押金提现",deposit,outRefundNo);
        return i;
    }

    @Override
    public EtRefund createRefund(EtOrder etOrder, BigDecimal refundAmount, BigDecimal appointmentFee, BigDecimal dispatchFee, BigDecimal manageFee, BigDecimal ridingFee, String outRefundNo,String type) {
        EtRefund etRefund = new EtRefund();
        etRefund.setAmount(refundAmount);
        etRefund.setReason(etOrder.getReason());
        etRefund.setAppointmentFee(appointmentFee);
        etRefund.setDispatchFee(dispatchFee);
        etRefund.setManageFee(manageFee);
        etRefund.setRidingFee(ridingFee);
        etRefund.setOrderNo(etOrder.getOrderNo());
        etRefund.setUserId(etOrder.getUserId());
        etRefund.setRefundNo(outRefundNo);
        etRefund.setType(type);
        if(type.equals(ServiceConstants.REFUND_TYPE_DEPOSIT)){
            etRefund.setItemDesc("押金退款");
        }else{
            StringBuilder itemDesc = new StringBuilder();
            if(ObjectUtil.isNotNull(appointmentFee) && !appointmentFee.equals(BigDecimal.ZERO)){
                itemDesc.append("预约费:").append(appointmentFee).append("元,");
            }
            if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
                itemDesc.append("调度费:").append(dispatchFee).append("元,");
            }
            if(ObjectUtil.isNotNull(manageFee) && !manageFee.equals(BigDecimal.ZERO)){
                itemDesc.append("停车点外调度费:").append(manageFee).append("元,");
            }
            if(ObjectUtil.isNotNull(ridingFee) && !ridingFee.equals(BigDecimal.ZERO)){
                itemDesc.append("骑行费:").append(ridingFee).append("元,");
            }
            etRefund.setItemDesc(itemDesc.toString());
        }
        return etRefund;
    }

    private void setAreaInfo(String sn, EtOrder etOrder) {
        AsDevice asDevice = deviceService.selectAsDeviceBySn(sn);
        EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(asDevice.getAreaId());
        etOrder.setAreaId(etOperatingArea.getAreaId());
        etOrder.setArea(etOperatingArea.getAreaName());
        etOrder.setSn(sn);
    }

    /**
     * 查询当前用户是否有正在进行中的订单
     */
    @Override
    public List<EtOrder> isInOrder(Long userId,String orderNo) {
        List<EtOrder> inOrder = etOrderMapper.isInOrder(userId, orderNo);
        for(EtOrder order:inOrder){
            EtFeeRule etFeeRule = etFeeRuleService.selectEtFeeRuleByRuleIdIncludeDelete(order.getRuleId());
            if(ObjectUtil.isNotNull(etFeeRule)){
                order.setRule(etFeeRule);
            }
            EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(order.getCouponId());
            if(ObjectUtil.isNotNull(etCoupon)){
                order.setCoupon(etCoupon);
            }
            // 骑行结束并且订单金额等于0，并且未支付
            if(ServiceConstants.ORDER_STATUS_RIDING_END.equals(order.getStatus()) && order.getTotalFee().compareTo(BigDecimal.ZERO) == 0 && order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT)){
                order.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
                order.setPayTime(new Date());
                order.setPayType(ServiceConstants.PAY_TYPE_SYS);
                order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                int updateEtOrder = etOrderMapper.updateEtOrder(order);
                if(updateEtOrder == 0){
                    throw new ServiceException("更新订单outTradeNo失败");
                }else {
                    log.info("【isInOrder接口】更新订单outTradeNo成功");
                }
            }
        }
        return inOrder;
    }

    /**
     * 查询当前用户是否有正在进行中的订单
     */
    @Override
    public List<EtOrder> isInAuditOrder(Long userId,String orderNo) {
        List<EtOrder> inOrder = etOrderMapper.isInAuditOrder(userId, orderNo);
        return inOrder;
    }
    /**
     * 查询当前车辆是否有正在进行中的订单
     */
    @Override
    public Boolean isInOrderBySn(String sn) {
        int inOrder = etOrderMapper.isInOrderBySn(sn);
        return inOrder > 0;
    }

    /**
     * 换车关锁
     *  1. 关锁
     *  2. 改变订单信息，sn,  保存使用过的sn
     *
     */
    @Override
    @Transactional
    @SneakyThrows
    public Boolean changeVehicleLock(String orderNo,String changeReason,String isBluetooth,String lon,String lat,String voltage,String newSn) {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(orderNo);
        if(!ServiceConstants.ORDER_STATUS_RIDING.equals(order.getStatus())){
            throw new ServiceException("该订单状态非骑行中");
        }
        String sn = order.getSn();
        if(StrUtil.isNotBlank(sn)){
            AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
            /** 2.发送命令*/
            if(!"true".equals(isBluetooth)){
                ResponseVo responseVo = asDeviceService.sendCommandWithResp(asDevice.getMac(), Token.getToken(), IotConstants.COMMAND_CLOSE + IotConstants.COMMAND_FREQUENCY_3600, "换车关锁",orderNo);
                if(responseVo.getCode()!=0){
                    log.info("【还车关锁】远程关锁失败");
                    throw new ServiceException("远程关锁失败");
                }
            }else{
                asDevice.setLongitude(lon);
                asDevice.setLatitude(lat);
                if(StrUtil.isNotBlank(voltage)){
                    BigDecimal divide = new BigDecimal(voltage).divide(new BigDecimal(10));
                    asDevice.setVoltage(divide.toString());
                    EtModel model = etModelService.selectEtModelByModelId(asDevice.getModelId());
                    Integer remainingMileage = 0;
                    if(StrUtil.isNotBlank(asDevice.getVoltage())){
                        remainingMileage = CommonUtil.getRemainingMileage(asDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
                    }
                    Integer electricQuantity = CommonUtil.getElectricQuantity(asDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
                    asDevice.setRemainingMileage(remainingMileage);
                    asDevice.setRemainingPower(electricQuantity.toString());
                }
                asDevice.setLastTime(DateUtils.getNowDate());
            }
            Boolean execute = transactionTemplate.execute(e -> {
                asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//换车关锁
                asDevice.setIsAdminUnlocking("0");
                int device = asDeviceMapper.updateAsDevice(asDevice);
                if(device==0){
                    log.info("【临时解锁】改变车辆状态失败");
                    throw new ServiceException("【临时锁车】改变车辆状态失败");
                }
                String usedSn = order.getUsedSn();
                if(StrUtil.isNotBlank(usedSn)){
                    usedSn = usedSn+","+sn;
                }else{
                    usedSn = sn;
                }
                order.setUsedSn(usedSn);
                order.setSn(newSn);
                order.setChangeReason(changeReason);
                int i = etOrderMapper.updateEtOrderByOrderNo(order);
                if(i==0){
                    log.info("【换车关锁】改变订单使用过的sn失败");
                    throw new ServiceException("【换车关锁】改变订单使用过的sn失败");
                }
                return Boolean.TRUE;
            });
            if(execute){
                return Boolean.TRUE;
            }else{
                return Boolean.FALSE;
            }
        }else{
            return Boolean.TRUE;
        }
    }

    /**
     * 换车开锁
     * 1. 开锁
     * 2. 更新订单的sn
     */
    @SneakyThrows
    @Override
    @Transactional
    public Boolean changeVehicleLockUnlocking(String orderNo, String newSn,String isBluetooth,String lon,String lat,String voltage) {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(orderNo);
        if(!ServiceConstants.ORDER_STATUS_RIDING.equals(order.getStatus())){
            throw new ServiceException("该订单状态非骑行中");
        }
        AsDevice newDevice = asDeviceMapper.selectAsDeviceBySn(newSn);
        if (ObjectUtil.isNull(newDevice)) {
            throw new ServiceException("设备不存在："+ newSn);
        }
        if(order.getAreaId() != newDevice.getAreaId()){
            throw new ServiceException("不同运营区，请勿操作");
        }
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            if(!"true".equals(isBluetooth)){
                ResponseVo responseVo = asDeviceService.sendCommandWithResp(newDevice.getMac(), token, IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_5, "换车开锁",orderNo);
                if(responseVo.getCode()!=0){
                    log.info("【换车开锁】远程关锁失败");
                    throw new ServiceException("换车开锁失败");
                }
            }else{
                newDevice.setLongitude(lon);
                newDevice.setLatitude(lat);
                if(StrUtil.isNotBlank(voltage)){
                    BigDecimal divide = new BigDecimal(voltage).divide(new BigDecimal(10));
                    newDevice.setVoltage(divide.toString());
                    EtModel model = etModelService.selectEtModelByModelId(newDevice.getModelId());
                    Integer remainingMileage = 0;
                    if(StrUtil.isNotBlank(newDevice.getVoltage())){
                        remainingMileage = CommonUtil.getRemainingMileage(newDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
                    }
                    Integer electricQuantity = CommonUtil.getElectricQuantity(newDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
                    newDevice.setRemainingMileage(remainingMileage);
                    newDevice.setRemainingPower(electricQuantity.toString());
                }
                newDevice.setLastTime(DateUtils.getNowDate());
            }
            newDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);
            newDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int i = asDeviceMapper.updateAsDevice(newDevice);
            if(i>0){
                log.info("管理员开锁，更新设备状态成功");
            }
            order.setSn(newSn);
            int i2 = etOrderMapper.updateEtOrderByOrderNo(order);
            if(i2==0){
                log.info("【换车开锁】更新订单sn失败");
                throw new ServiceException("【换车开锁】更新订单sn失败");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("换车开锁失败");
        return Boolean.TRUE;
    }

    /**
     * 使用优惠券
     *      1. 查询订单，
     *      2. 计算使用优惠券后的金额，
     *      3. 更新优惠券状态
     *      4. 更新订单状态
     */
    @Override
    public int useCoupon(String orderNo, Long logId) {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(orderNo);
        if(ObjectUtil.isNotNull(order)){
            order = calculateCoupon(order,logId);
        }else{
            throw new ServiceException("订单不存在");
        }
        order.setLogId(logId);
        int updateEtOrder = etOrderMapper.updateEtOrder(order);
        log.info("【使用优惠券】更新订单状态成功");
        return updateEtOrder;
    }

    /**
     * 使用优惠券后的价格
     *  1. 先判断是否过期？并且是否是未使用的状态
     *  2. 再判断是哪种类型的优惠券，
     *     如果是
     *     时间卡：骑行价格直接改成0
     *     贵宾卡：所有价格改成0
     *     折扣卡：总金额 * 折扣比例  次数减一
     *     抵用券：总金额 - 抵用金额  次数减一
     */
    private EtOrder calculateCoupon(EtOrder order,Long logId) {
        Long userId = order.getUserId();
        AsUser asUser = asUserMapper.selectUserById(userId);
        log.info("【用户】：【{}】",JSON.toJSON(asUser));
        EtCouponUserLog etCouponUserLog1 = etCouponClaimLogMapper.selectEtCouponClaimLogByLogId(logId);
        log.info("【使用优惠券】优惠券信息：【{}】",JSON.toJSON(etCouponUserLog1));
        EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(etCouponUserLog1.getCouponId());
        log.info("【优惠券】优惠券：【{}】",JSON.toJSON(etCoupon));
        if(etCouponUserLog1 == null){
            throw new ServiceException("用户没有优惠券");
        }
        if(etCoupon == null){
            throw new ServiceException("优惠券不存在");
        }
        if(ObjectUtil.isNotNull(asUser.getExpirationTime()) && asUser.getExpirationTime().compareTo(new Date()) < 0){
            throw new ServiceException("会员已过期");
        }

        if((etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_DISCOUNT_CARD) || etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VOUCHER))
                && isCouponExpired(etCouponUserLog1)){
            throw new ServiceException("优惠券已过期");
        }

        if(etCouponUserLog1.getLimitNum() == 0){
            throw new ServiceException("可用次数不足");
        }

        if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_TIME_CARD)){
            order.setPayFee(formatPayFee(order.getTotalFee().subtract(order.getRidingFee())));//会员
            order.setRidingFee(BigDecimal.ZERO);
        }else if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VIP_CARD)){//贵宾卡
            order.setPayFee(BigDecimal.ZERO);
        }else if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_DISCOUNT_CARD)){//折扣卡
            order.setPayFee(formatPayFee(order.getTotalFee().multiply(etCoupon.getDiscountPercent()).setScale(2, RoundingMode.HALF_UP)));
        }else if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VOUCHER)){//抵用券
            order.setPayFee(formatPayFee(order.getTotalFee().subtract(etCoupon.getDiscountAmount())));
        }else{
            throw new ServiceException("优惠券类型错误");
        }

        return order;
    }

    /**
     * 格式化金额方法：保留两位小数，如果金额为负数则返回0.
     *
     * @param amount 需要格式化的金额
     * @return 格式化后的金额
     */
    public static BigDecimal formatPayFee(BigDecimal amount) {
        return amount.max(BigDecimal.ZERO).setScale(2, RoundingMode.HALF_UP);
    }


    /** 判断优惠券是否过期 */
    private boolean isCouponExpired(EtCouponUserLog etCouponUserLog) {
        if(etCouponUserLog!=null){
            return etCouponUserLog.getExpirationTime().compareTo(new Date()) < 0;
        }else{
            return true;
        }
    }
}
