package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtReconciliationMapper;
import com.ruoyi.system.domain.EtReconciliation;
import com.ruoyi.system.service.IEtReconciliationService;

/**
 * 平台对账Service业务层处理
 * 
 * @author qzz
 * @date 2024-10-03
 */
@Service
public class EtReconciliationServiceImpl implements IEtReconciliationService 
{
    @Autowired
    private EtReconciliationMapper etReconciliationMapper;

    /**
     * 查询平台对账
     * 
     * @param reconciliationId 平台对账主键
     * @return 平台对账
     */
    @Override
    public EtReconciliation selectEtReconciliationByReconciliationId(Long reconciliationId)
    {
        return etReconciliationMapper.selectEtReconciliationByReconciliationId(reconciliationId);
    }

    /**
     * 查询平台对账列表
     * 
     * @param etReconciliation 平台对账
     * @return 平台对账
     */
    @Override
    public List<EtReconciliation> selectEtReconciliationList(EtReconciliation etReconciliation)
    {
        return etReconciliationMapper.selectEtReconciliationList(etReconciliation);
    }

    /**
     * 新增平台对账
     * 
     * @param etReconciliation 平台对账
     * @return 结果
     */
    @Override
    public int insertEtReconciliation(EtReconciliation etReconciliation)
    {
        etReconciliation.setCreateTime(DateUtils.getNowDate());
        return etReconciliationMapper.insertEtReconciliation(etReconciliation);
    }

    /**
     * 修改平台对账
     * 
     * @param etReconciliation 平台对账
     * @return 结果
     */
    @Override
    public int updateEtReconciliation(EtReconciliation etReconciliation)
    {
        return etReconciliationMapper.updateEtReconciliation(etReconciliation);
    }

    /**
     * 批量删除平台对账
     * 
     * @param reconciliationIds 需要删除的平台对账主键
     * @return 结果
     */
    @Override
    public int deleteEtReconciliationByReconciliationIds(Long[] reconciliationIds)
    {
        return etReconciliationMapper.deleteEtReconciliationByReconciliationIds(reconciliationIds);
    }

    /**
     * 删除平台对账信息
     * 
     * @param reconciliationId 平台对账主键
     * @return 结果
     */
    @Override
    public int deleteEtReconciliationByReconciliationId(Long reconciliationId)
    {
        return etReconciliationMapper.deleteEtReconciliationByReconciliationId(reconciliationId);
    }
}
