package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.enums.PayChannel;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.PaymentResult;
import com.ruoyi.common.pay.tm.TmPayService;
import com.ruoyi.common.pay.tm.enums.PayStatus;
import com.ruoyi.common.pay.tm.enums.RefundStatus;
import com.ruoyi.common.pay.tm.vo.RefundInfo;
import com.ruoyi.common.pay.tm.vo.TmTradeInfo;
import com.ruoyi.common.pay.wx.Payable;
import com.ruoyi.common.pay.wx.RefundAble;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.ChannelVO;
import com.ruoyi.system.domain.EtAreaDept;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.EtRefund;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.mapper.EtAreaDeptMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.JsapiServiceExtension;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.RefundService;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.QueryByOutRefundNoRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferRequest;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 微信支付服务
 * @author 辉
 * 2024/3/11
 */
@Service
@Slf4j
public class WxPayService implements IWxPayService {

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private RedisLock redisLock;

    @Resource
    private EtAreaDeptMapper etAreaDeptMapper;

    @Autowired
    private ISysDeptService deptService;

    @Resource
    private AsUserMapper asUserMapper;

    @Autowired
    private TmPayService tmPayService;

    @Autowired
    private EtChannelService etChannelService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IEtRefundService etRefundService;

    private static final String CNY = "CNY";

    private static final String PREPAY_LOCK = "prepay:";

    @Override
    public PrepayWithRequestPaymentResponse prepayWithRequestPayment(String payType,EtOrder order) {
        String billNo = order.getOrderNo();
        // 创建订单
        AsUser user = asUserService.selectUserById(order.getUserId());
        try {
            List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", order.getAreaId()));
            if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
                EtAreaDept etAreaDept = areaId1.get(0);
                SysDept sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
                log.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
                if(ObjectUtil.isNull(sysDept)){
                    throw new ServiceException("没有运营商：【"+etAreaDept.getDeptId()+"】");
                }

                Long payChannel = sysDept.getPayChannel();
                if(ObjectUtil.isNull(payChannel)){
                    throw new ServiceException("运营商【"+sysDept.getDeptName()+"】没有支付渠道");
                }
                ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(payChannel);
                log.info("支付渠道------"+JSON.toJSON(channelVO));
                String outTradeNo = null;
                if(PayChannel.CT_WX.equalsCode(channelVO.getCode()) || PayChannel.YS_WX.equalsCode(channelVO.getCode())){
                    outTradeNo = IdUtils.getOrderNo("wx");
                }else if(PayChannel.TM_WX.equalsCode(channelVO.getCode())){
                    if(payType.equals(ServiceConstants.ORDER_TYPE_RIDING)){
                        outTradeNo = IdUtils.getOrderNo("tmwx");
                    } else if(payType.equals(ServiceConstants.BUSINESS_TYPE_DEPOSIT)){
                        outTradeNo = IdUtils.getOrderNo("tmyj");
                    }else if(payType.equals(ServiceConstants.ORDER_TYPE_COUPON)){
                        outTradeNo = IdUtils.getOrderNo("tmyhj");
                    }else{
                        throw new ServiceException("暂不支持该支付场景");
                    }
                }
                String type = order.getType();
                String description = type.equals(ServiceConstants.ORDER_TYPE_RIDING) ? "骑行订单-"+billNo : "押金充值-"+billNo;

                EtOrder order1 = new EtOrder();
                order1.setOutTradeNo(outTradeNo);
                order1.setOrderId(order.getOrderId());

                order1.setPayChannel(payChannel);
                int updateEtOrder = etOrderService.updateEtOrder(order1);
                if(updateEtOrder == 0){
                    throw new ServiceException("更新订单outTradeNo失败");
                }

                if(PayChannel.CT_WX.equalsCode(channelVO.getCode()) || PayChannel.YS_WX.equalsCode(channelVO.getCode())){
                    log.info("----------{}-------------","微信官方支付");
                    PrepayRequest request = new PrepayRequest();
                    request.setAmount(getAmount(order.getPayFee()));
                    request.setOutTradeNo(outTradeNo);
                    request.setAppid(sysDept.getAppid());
                    request.setMchid(channelVO.getMerchantId());
                    request.setAttach(JSON.toJSONString(new AttachVo(payType,user.getUserId(), "")));
                    request.setDescription(description);
                    request.setNotifyUrl(channelVO.getNotifyUrl());
                    request.setPayer(getPayer(user.getWxopenid()));
                    JsapiServiceExtension jsapiServiceExtension = getJsapiServiceExtension(channelVO);
                    PrepayWithRequestPaymentResponse res = jsapiServiceExtension.prepayWithRequestPayment(request);

                    if(StrUtil.isNotBlank(order.getOutTradeNo())){
                        String tradeNo = order.getOutTradeNo();
                        // 关闭订单
                        CloseOrderRequest closeOrderRequest = new CloseOrderRequest();
                        closeOrderRequest.setMchid(channelVO.getMerchantId());
                        closeOrderRequest.setOutTradeNo(tradeNo);
                        jsapiServiceExtension.closeOrder(closeOrderRequest);
                    }
                    return res;
                }else if(PayChannel.TM_WX.equalsCode(channelVO.getCode())){
                    log.info("----------{}-------------","太米微信支付");

                    if(StrUtil.isNotBlank(order.getOutTradeNo())){
                        // 关闭订单
                        tmPayService.closeOrder(channelVO,order.getOutTradeNo());
                    }
                    Payable payable = new Payable();
                    payable.setAmount(order.getPayFee().multiply(new BigDecimal(100)).longValue());
                    payable.setOutTradeNo(outTradeNo);
                    payable.setAttach(JSON.toJSONString(new AttachVo(payType,user.getUserId(), "")));
                    payable.setDescription(description);
                    payable.setOpenid(user.getWxopenid());
                    payable.setAppid(channelVO.getAppid());
                    PrepayWithRequestPaymentResponse res = tmPayService.pay(channelVO,payable);
                    return res;
                }else{
                    throw new ServiceException("支付渠道【"+channelVO.getCode()+"】暂不支持");
                }
            }else{
                log.info("区域：【{}】没有绑定运营商",order.getAreaId());
                throw new ServiceException("区域：【"+order.getAreaId()+"】没有绑定运营商");
            }
        } finally {
            redisLock.unlock(PREPAY_LOCK + billNo);
        }
    }

    @Override
    public PrepayWithRequestPaymentResponse prepayWithRequestPayment(EtOrder order) {
        String billNo = order.getOrderNo();
        // 创建订单
        AsUser user = asUserService.selectUserById(order.getUserId());
        try {
            List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", order.getAreaId()));
            if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
                EtAreaDept etAreaDept = areaId1.get(0);
                SysDept sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
                log.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
                if(ObjectUtil.isNull(sysDept)){
                    throw new ServiceException("没有运营商：【"+etAreaDept.getDeptId()+"】");
                }

                String description = "优惠券订单-"+billNo;

                String outTradeNo = IdUtils.getOrderNo("wx");
                EtOrder order1 = new EtOrder();
                order1.setOutTradeNo(outTradeNo);
                order1.setOrderId(order.getOrderId());

                Long payChannel = sysDept.getPayChannel();
                if(ObjectUtil.isNull(payChannel)){
                    throw new ServiceException("运营商【"+sysDept.getDeptName()+"】没有支付渠道");
                }
                order1.setPayChannel(payChannel);
                int updateEtOrder = etOrderService.updateEtOrder(order1);
                if(updateEtOrder == 0){
                    throw new ServiceException("更新订单outTradeNo失败");
                }

                ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(payChannel);
                if(PayChannel.CT_WX.equalsCode(channelVO.getCode()) || PayChannel.YS_WX.equalsCode(channelVO.getCode())){
                    log.info("----------{}-------------","微信官方支付");
                    PrepayRequest request = new PrepayRequest();
                    request.setAmount(getAmount(order.getPayFee()));
                    request.setOutTradeNo(outTradeNo);
                    request.setAppid(sysDept.getAppid());
                    request.setMchid(channelVO.getMerchantId());
                    request.setAttach(JSON.toJSONString(new AttachVo(order.getType(),user.getUserId(), "")));
                    request.setDescription(description);
                    request.setNotifyUrl(channelVO.getNotifyUrl());
                    request.setPayer(getPayer(user.getWxopenid()));
                    JsapiServiceExtension jsapiServiceExtension = getJsapiServiceExtension(channelVO);
                    PrepayWithRequestPaymentResponse res = jsapiServiceExtension.prepayWithRequestPayment(request);

                    if(StrUtil.isNotBlank(order.getOutTradeNo())){
                        String tradeNo = order.getOutTradeNo();
                        // 关闭订单
                        CloseOrderRequest closeOrderRequest = new CloseOrderRequest();
                        closeOrderRequest.setMchid(channelVO.getMerchantId());
                        closeOrderRequest.setOutTradeNo(tradeNo);
                        jsapiServiceExtension.closeOrder(closeOrderRequest);
                    }

                    return res;
                }else if(PayChannel.TM_WX.equalsCode(channelVO.getCode())){
                    log.info("----优惠券------{}-------------","太米微信支付");

                    // 获取JSAPI所需参数
                    if(StrUtil.isNotBlank(order.getOutTradeNo())){
                        // 关闭订单
                        tmPayService.closeOrder(channelVO,order.getOutTradeNo());
                    }

                    Payable payable = new Payable();
                    payable.setAmount(order.getPayFee().longValue());
                    payable.setOutTradeNo(outTradeNo);
                    payable.setAttach(JSON.toJSONString(new AttachVo(order.getType(),user.getUserId(), "")));
                    payable.setDescription(description);
                    payable.setOpenid(user.getWxopenid());
                    PrepayWithRequestPaymentResponse res = tmPayService.pay(channelVO,payable);
                    return res;
                }else{
                    throw new ServiceException("支付渠道【"+channelVO.getCode()+"】暂不支持");
                }
            }else{
                log.info("区域：【{}】没有绑定运营商",order.getAreaId());
                throw new ServiceException("区域：【"+order.getAreaId()+"】没有绑定运营商");
            }
        } finally {
            redisLock.unlock(PREPAY_LOCK + billNo);
        }
    }


    private JsapiServiceExtension getJsapiServiceExtension(ChannelVO channelVO) {
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(channelVO.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(channelVO.getPrivateKeyPath())
                .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                .apiV3Key(channelVO.getApiV3Key())
                .build();
        // 初始化服务
        return new JsapiServiceExtension
                .Builder()
                .config(config)
                .signType("RSA") // 不填默认为RSA
                .build();
    }

    /**
     * 根据订单号查询订单
     * @param outTradeNo 商户订单号
     * @return
     */
    @Override
    public PaymentResult queryOrderByOutTradeNo(String outTradeNo) {
        EtOrder order = etOrderService.selectEtOrderByOutTradeNo(outTradeNo);
        ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(order.getPayChannel());
        PaymentResult paymentResult1 = new PaymentResult();
        if(PayChannel.CT_WX.equalsCode(channelVO.getCode()) || PayChannel.YS_WX.equalsCode(channelVO.getCode())){
            QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
            request.setMchid(channelVO.getMerchantId());
            request.setOutTradeNo(outTradeNo);
            JsapiService jsapiService = getJsapiService(channelVO);
            Transaction transaction = jsapiService.queryOrderByOutTradeNo(request);
            log.info("微信查询订单信息outTradeNo={}-----【{}】",outTradeNo,JSON.toJSON(transaction));
            paymentResult1.setTransaction(transaction);
            return paymentResult1;
        }else if(PayChannel.TM_WX.equalsCode(channelVO.getCode())){
            TmTradeInfo tmTradeInfo = tmPayService.orderQuery(channelVO, order.getOutTradeNo());
            paymentResult1.setTradeInfo(tmTradeInfo);
            return paymentResult1;
        }else{
            throw new ServiceException("支付渠道【"+channelVO.getCode()+"】暂不支持");
        }
    }

    /**
     * 关闭订单
     * @param outTradeNo 商户订单号
     * @return
     */
    @Override
    public boolean closeOrder(String outTradeNo,ChannelVO channelVO) {
        JsapiServiceExtension jsapiServiceExtension = getJsapiServiceExtension(channelVO);
        // 关闭订单
        CloseOrderRequest closeOrderRequest = new CloseOrderRequest();
        closeOrderRequest.setMchid(channelVO.getMerchantId());
        closeOrderRequest.setOutTradeNo(outTradeNo);
        jsapiServiceExtension.closeOrder(closeOrderRequest);
        return true;
    }

    /**
     * 根据订单号查询订单
     * @param orderNo 商户订单号
     * @return
     */
    @Override
    public Boolean queryResultByOrderNo(String orderNo) {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(order.getPayChannel());
        if(PayChannel.CT_WX.equalsCode(channelVO.getCode()) || PayChannel.YS_WX.equalsCode(channelVO.getCode())){
            QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
            request.setMchid(channelVO.getMerchantId());
            if(StrUtil.isBlank(order.getOutTradeNo())){
                return false;
            }
            request.setOutTradeNo(order.getOutTradeNo());
            JsapiService jsapiService = getJsapiService(channelVO);
            Transaction transaction = jsapiService.queryOrderByOutTradeNo(request);
            // 订单未支付并且微信支付结果是成功的情况下，更新订单状态和用户余额
            if(transaction.getTradeState().equals(Transaction.TradeStateEnum.SUCCESS)){
                if(order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT)){
                    handlePaySuccess(order, transaction.getSuccessTime());
                }
                return true;
            }
        }else if(PayChannel.TM_WX.equalsCode(channelVO.getCode())){
            TmTradeInfo tmTradeInfo = tmPayService.orderQuery(channelVO, order.getOutTradeNo());
            if(PayStatus.isSuccess(tmTradeInfo.getPayStatus().getCode())) {
                if(order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT)){
                    handlePaySuccess(order, tmTradeInfo.getPayTime());
                }
                return true;
            }
        }else{
            throw new ServiceException("支付渠道【"+channelVO.getCode()+"】暂不支持");
        }
        return false;
    }

    private void handlePaySuccess(EtOrder order, String payTime) {
        order.setPaid("1");
        order.setPayTime(DateUtils.parseTime(payTime));
        order.setPayType(ServiceConstants.PAY_TYPE_WX);
        log.info("【主动查询】押金支付");
        order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        if(ServiceConstants.ORDER_TYPE_RIDING.equals(order.getType())){
            order.setMark("主动查询-骑行支付");
        }else{
            order.setMark("押金支付");
            // 更新用户余额
            AsUser asUser = asUserMapper.selectUserById(order.getUserId());
            asUser.setBalance(order.getTotalFee());
            int updateUser = asUserMapper.updateUser(asUser);
            if(updateUser==0){
                log.error("【微信支付回调】更新用户押金失败");
                throw new ServiceException("【微信支付回调】更新用户押金失败");
            }
        }
        int updateEtOrder = etOrderService.updateEtOrder(order);
        if(updateEtOrder==0){
            log.error("【微信支付回调】更新订单信息失败");
            throw new ServiceException("【微信支付回调】更新订单信息失败");
        }
    }

    private JsapiService getJsapiService(ChannelVO channelVO) {
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(channelVO.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(channelVO.getPrivateKeyPath())
                .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                .apiV3Key(channelVO.getApiV3Key())
                .build();
        // 初始化服务
        return new JsapiService.Builder().config(config).build();
    }

    /**
     * 退款
     * @param etOrder 原订单对象
     * @param reason 退款原因
     * @param amount 退款金额
     * @param outRefundNo 退款订单号
     */
    @Override
    public void refund(EtOrder etOrder,String reason,BigDecimal amount,String outRefundNo) {
        if(!etOrder.getStatus().equals(ServiceConstants.ORDER_STATUS_ORDER_END)) throw new ServiceException("订单状态异常");
        ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(etOrder.getPayChannel());
        if (channelVO == null) {
            throw new ServiceException("支付渠道不存在: " + etOrder.getPayChannel());
        }

        if(PayChannel.CT_WX.equalsCode(channelVO.getCode()) || PayChannel.YS_WX.equalsCode(channelVO.getCode())){
            log.info("----------{}-------------","微信官方退款");
            CreateRequest request = new CreateRequest();
            request.setOutTradeNo(etOrder.getOutTradeNo());
            request.setOutRefundNo(outRefundNo);
            request.setReason(reason);
            request.setAmount(getAmountReq(etOrder.getTotalFee(),amount));
            request.setNotifyUrl(channelVO.getRefundNotifyUrl());
            log.info("【退款】请求微信参数:【{}】",JSON.toJSONString(request));
            RefundService refundService = getRefundService(channelVO);
            Refund refund = refundService.create(request);
            log.info("【退款】微信返回结果:【{}】",JSON.toJSONString(refund));
        }else if(PayChannel.TM_WX.equalsCode(channelVO.getCode())){
            log.info("----------{}-------------","太米微信退款");
            RefundAble refundAble = new RefundAble();
            refundAble.setOutTradeNo(etOrder.getOutTradeNo());
            refundAble.setOutRefundNo(outRefundNo);
            refundAble.setReason(reason);
            refundAble.setAmount(amount.multiply(new BigDecimal(100)).longValue());
            log.info("【退款】请求太米参数:【{}】",JSON.toJSONString(refundAble));
            RefundInfo refund = tmPayService.refund(channelVO, refundAble);
            ServiceUtil.assertion(!RefundStatus.isSuccess(refund.getRefundStatus().getCode()), "发起退款失败:" + refund.getRefundMessage());
            scheduledExecutorService.schedule(() -> {
                handleRefundSuccess(outRefundNo);
            }, 0, TimeUnit.SECONDS);
        }else{
            throw new ServiceException("支付渠道【"+channelVO.getCode()+"】暂不支持");
        }
    }

    /**
     * 退款成功后的处理
     * @param refundNo 退款订单号
     */
    private void handleRefundSuccess(String refundNo) {
        EtRefund etRefund = new EtRefund();
        etRefund.setRefundNo(refundNo);
        etRefund.setRefundResult(Constants.SUCCESS2);
        int i = etRefundService.updateEtRefundByRefundNo(etRefund);
        if(i==0){
            log.error("【通联退款】更新退款单失败");
            throw new ServiceException("【通联退款】更新退款单失败");
        }
    }

    private RefundService getRefundService(ChannelVO channelVO){
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(channelVO.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(channelVO.getPrivateKeyPath())
                .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                .apiV3Key(channelVO.getApiV3Key())
                .build();
        // 初始化服务
        return new RefundService.Builder().config(config).build();
    }

    /**
     * 发起商家转账
     * @param sysDept 运营商对象
     * @param batchNo 批次号
     * @param batchName 批次名称
     * @param batchRemark 转账说明
     * @param totalAmount 转账总金额
     * @param totalNum 转账总笔数
     * @param transferDetailInputs 转账明细列表
     */
    @Override
    public InitiateBatchTransferResponse transfer(SysDept sysDept,String batchNo,String batchName,String batchRemark,BigDecimal totalAmount,Integer totalNum,List<TransferDetailInput> transferDetailInputs) {
//        SysDept sysDept = getDeptObjByAreaId(areaId);
        InitiateBatchTransferRequest request = new InitiateBatchTransferRequest();
        request.setAppid(sysDept.getAppid());
        request.setOutBatchNo(batchNo);
        request.setBatchName(batchName);
        request.setBatchRemark(batchRemark);
        request.setTotalAmount(totalAmount.multiply(new BigDecimal(100)).longValue());
        request.setTotalNum(totalNum);
        request.setTransferDetailList(transferDetailInputs);
        log.info("【发起商家转账】请求微信参数:【{}】",JSON.toJSONString(request));
//        TransferBatchService transferService = getTransferService(sysDept);
//        InitiateBatchTransferResponse initiateBatchTransferResponse = transferService.initiateBatchTransfer(request);
//        log.info("【发起商家转账】微信返回结果:【{}】",JSON.toJSONString(initiateBatchTransferResponse));
//        return initiateBatchTransferResponse;
        return null;
    }

//    private TransferBatchService getTransferService(SysDept sysDept){
//        // 初始化商户配置
//        Config config = new RSAAutoCertificateConfig.Builder()
//                .merchantId(sysDept.getMerchantId())
//                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
//                .privateKeyFromPath(sysDept.getPrivateKeyPath())
//                .merchantSerialNumber(sysDept.getMerchantSerialNumber())
//                .apiV3Key(sysDept.getApiV3Key())
//                .build();
//        // 初始化服务
//        return new TransferBatchService.Builder().config(config).build();
//    }


    /**
     * 根据运营区id获取运营商对象
     * @param areaId
     * @return
     */
    @Override
    public SysDept getDeptObjByAreaId(Long areaId) {
        SysDept sysDept;
        List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", areaId));
        if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
            EtAreaDept etAreaDept = areaId1.get(0);
             sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
            log.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
        }else{
            log.info("区域：【{}】没有绑定运营商",areaId);
            return null;
        }
        return sysDept;
    }

    @Override
    public Refund queryByOutRefundNo(String outRefundNo) {
        EtRefund refund = etRefundService.selectEtRefundByRefundNo(outRefundNo);
        if (refund == null) {
            throw new ServiceException("没有找到退款记录: " + outRefundNo);
        }
        EtOrder order = etOrderService.selectEtOrderByOrderNo(refund.getOrderNo());
        if (order == null) {
            throw new ServiceException("订单不存在: " + refund.getOrderNo());
        }
        ChannelVO channelVO = etChannelService.selectSmChannelByChannelId(order.getPayChannel());
        if (channelVO == null) {
            throw new ServiceException("支付渠道不存在: " + order.getPayChannel());
        }
        QueryByOutRefundNoRequest request = new QueryByOutRefundNoRequest();
        request.setOutRefundNo(outRefundNo);
        RefundService refundService = getRefundService(channelVO);
        return refundService.queryByOutRefundNo(request);
    }

//    /** 请求分账API */
//    public OrdersEntity createOrder(SysDept sysDept,String transactionId,List<CreateOrderReceiver> receivers) {
//        CreateOrderRequest request = new CreateOrderRequest();
//        request.setAppid(sysDept.getAppid());
//        request.setTransactionId(transactionId);// 微信订单号
//        request.setOutOrderNo(IdUtils.getOrderNo("fz"));// 商户系统内部分账单号
//        request.setReceivers(receivers);
//        request.setUnfreezeUnsplit(true);
//        ProfitsharingService profitsharingService = getProfitsharingService(sysDept);
//        return profitsharingService.createOrder(request);
//    }
//
//    /** 添加分账接收方 */
//    @Override
//    public AddReceiverResponse addReceiver(String wxopenid,Long deptId,String type) {
//        SysDept sysDept = deptService.selectDeptById(deptId);
//        AddReceiverRequest request = new AddReceiverRequest();
//        request.setAppid(sysDept.getAppid());
//        if(type.equals(ServiceConstants.PROFITSHARING_TYPE_PLATFORM)){
//            request.setType(ReceiverType.MERCHANT_ID);
//        }else{
//            request.setType(ReceiverType.PERSONAL_OPENID);
//        }
//        request.setAccount(wxopenid);
//        request.setRelationType(ReceiverRelationType.PARTNER);
//        ProfitsharingService profitsharingService = getProfitsharingService(sysDept);
//        return profitsharingService.addReceiver(request);
//    }

//    /** 删除分账接收方 */
//    public DeleteReceiverResponse deleteReceiver(String wxopenid,Long deptId,String type) {
//        SysDept sysDept = deptService.selectDeptById(deptId);
//        DeleteReceiverRequest request = new DeleteReceiverRequest();
//        request.setAppid(sysDept.getAppid());
//        if(type.equals(ServiceConstants.PROFITSHARING_TYPE_PLATFORM)){
//            request.setType(ReceiverType.MERCHANT_ID);
//        }else{
//            request.setType(ReceiverType.PERSONAL_OPENID);
//        }
//        request.setAccount(wxopenid);
//        ProfitsharingService profitsharingService = getProfitsharingService(sysDept);
//        return profitsharingService.deleteReceiver(request);
//    }

//    private ProfitsharingService getProfitsharingService(SysDept sysDept) {
//        Config config = new RSAAutoCertificateConfig.Builder()
//                        .merchantId(sysDept.getMerchantId())
//                        // 使用 com.wechat.pay.java.core.util中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
//                        .privateKeyFromPath(sysDept.getPrivateKeyPath())
//                        .merchantSerialNumber(sysDept.getMerchantSerialNumber())
//                        .apiV3Key(sysDept.getApiV3Key())
//                        .build();
//        // 初始化服务
//        return new ProfitsharingService.Builder().config(config).build();
//    }


//    /**
//     * 验签并解析
//     * @param request 请求
//     * @param body 请求体
//     * @param clazz 返回值类型
//     */
//    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
//        // 构造 RequestParam
//        RequestParam requestParam = new RequestParam.Builder()
//                .serialNumber(request.getHeader("Wechatpay-Serial"))
//                .nonce(request.getHeader("Wechatpay-Nonce"))
//                .signature(request.getHeader("Wechatpay-Signature"))
//                .timestamp(request.getHeader("Wechatpay-Timestamp"))
//                .body(body)
//                .build();
//        // 验签
//        return notificationParser.parse(requestParam, clazz);
//    }

    private Payer getPayer(String openId) {
        Payer payer = new Payer();
        payer.setOpenid(openId);
        return payer;
    }

    private Amount getAmount(BigDecimal money) {
        Amount amount = new Amount();
        amount.setTotal(money.multiply(new BigDecimal(100)).intValue());
        amount.setCurrency(CNY);
        return amount;
    }

    private AmountReq getAmountReq(BigDecimal total,BigDecimal rerundAmount) {
        AmountReq amount = new AmountReq();
        amount.setTotal(total.multiply(new BigDecimal(100)).longValue());
        amount.setRefund(rerundAmount.multiply(new BigDecimal(100)).longValue());
        amount.setCurrency(CNY);
        return amount;
    }


}
